#!/usr/bin/env python

import os
import sys
from code import InteractiveConsole
from atexit import register
from canari.pkgutils.transform import TransformDistribution

from common import canari_main, fix_pypath, fix_binpath, import_package, pushd
from framework import SubCommand, Argument
from canari.config import config
from canari.maltego.utils import highlight, console_message, local_transform_runner
import canari


__author__ = 'Nadeem Douba'
__copyright__ = 'Copyright 2012, Canari Project'
__credits__ = []

__license__ = 'GPL'
__version__ = '0.5'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'


class ShellCommand(object):

    def __init__(self, mod):
        self.mod = mod
        self.sudoargs = ['sudo'] + list(sys.argv)

    def __call__(self, value, *args, **kwargs):
        if os.name == 'posix' and hasattr(self.mod.dotransform, 'privileged') and os.geteuid():
            print highlight("Need to be root to run this transform... sudo'ing...", 'green', True)
            os.execvp('sudo', self.sudoargs)
            return

        local_transform_runner(self.mod, value, kwargs, list(args), config, message_writer=console_message)


class MtgConsole(InteractiveConsole):

    def __init__(self, package):
        package = import_package(package)
        transforms = dict(dir=dir)
        for name, mod in package.__dict__.iteritems():
            if getattr(mod, 'dotransform', ''):
                transforms[name] = ShellCommand(mod)
        InteractiveConsole.__init__(self, locals=transforms)
        self.init_history(os.path.expanduser('~/.mtgsh_history'))

    def init_history(self, history_file):
        try:
            import readline
            readline.parse_and_bind('tab: complete')
            if hasattr(readline, "read_history_file"):
                try:
                    readline.read_history_file(history_file)
                except IOError:
                    pass
                register(lambda h: readline.write_history_file(h), history_file)
        except ImportError:
            pass


@SubCommand(
    canari_main,
    help='Creates a Canari debug shell for the specified transform package.',
    description='Creates a Canari debug shell for the specified transform package.'
)
@Argument(
    'package',
    metavar='<package name>',
    help='The name of the canari package you wish to load local transform from for the Canari shell session.'
)
@Argument(
    '-w',
    '--working-dir',
    metavar='[working dir]',
    default=None,
    help="the path that will be used as the working directory for "
         "the transforms being executed in the shell (default: ~/.canari/)"
)
def shell(opts):

    fix_binpath(config['default/path'])
    fix_pypath()

    if not opts.package.endswith('transforms'):
        opts.package = '%s.transforms' % opts.package

    try:
        t = TransformDistribution(opts.package)
        with pushd(opts.working_dir or t.default_prefix):
            mtg_console = MtgConsole(opts.package)
            mtg_console.interact(highlight('Welcome to Canari %s.' % canari.__version__, 'green', True))
    except ValueError, e:
        print str(e)
        exit(-1)



#!/usr/bin/env python

from common import canari_main
from framework import SubCommand


__author__ = 'Nadeem Douba'
__copyright__ = 'Copyright 2012, Canari Project'
__credits__ = []

__license__ = 'GPL'
__version__ = '0.3'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'


@SubCommand(
    canari_main,
    help='Show banner of Canari framework that is currently active.',
    description='Show banner of Canari framework that is currently active.'
)
def banner(args):
    print """
    Your running ...
        dBBBP dBBBBBb     dBBBBb dBBBBBb   dBBBBBb    dBP    dBBBb   dBBBb
                   BB        dBP      BB       dBP
      dBP      dBP BB   dBP dBP   dBP BB   dBBBBK   dBP       dBP     dBP
     dBP      dBP  BB  dBP dBP   dBP  BB  dBP  BB  dBP       dBP     dBP
    dBBBBP   dBBBBBBB dBP dBP   dBBBBBBB dBP  dB' dBP       dBP dBP dBP
                                            ... http://canariproject.com
    """


#!/usr/bin/env python

from argparse import ArgumentParser

__author__ = 'Nadeem Douba'
__copyright__ = 'Copyright 2012, Canari Project'
__credits__ = ['Nadeem Douba']

__license__ = 'GPL'
__version__ = '0.1'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'


__all__ = [
    'Command',
    'SubCommand',
    'Argument'
]


class Command(object):
    def __init__(self, *args, **kwargs):
        self.parser = ArgumentParser(*args, **kwargs)

    def __call__(self, func, *args, **kwargs):
        if hasattr(self.parser, 'parser_args'):
            for a, k in func.parser_args:
                self.parser.add_argument(*a, **k)

        def _func(args_=None, namespace=None):
            return func(self.parser.parse_args(args_, namespace))

        _func.parser = self.parser
        return _func


class SubCommand(object):
    def __init__(self, parent, *args, **kwargs):
        if not hasattr(parent, 'subparsers'):
            parent.subparsers = parent.parser.add_subparsers(
                title='subcommands',
                description='valid subcommands',
                help='additional help'
            )
        self.args = args
        self.kwargs = kwargs
        self.subparsers = parent.subparsers
        self.parser = None

    def __call__(self, func, *args, **kwargs):
        if not self.args:
            self.args = [func.__name__.replace('_', '-')]
        self.parser = self.subparsers.add_parser(*self.args, **self.kwargs)
        if hasattr(func, 'parser_args'):
            for a, k in func.parser_args:
                self.parser.add_argument(*a, **k)
        func.parser = self.parser
        func.parser.set_defaults(command_function=func)
        return func


class Argument(object):
    def __init__(self, *args, **kwargs):
        self.args = args
        self.kwargs = kwargs

    def __call__(self, func, *args, **kwargs):
        if not hasattr(func, 'parser_args'):
            func.parser_args = []
        func.parser_args.insert(0, (self.args, self.kwargs))
        return func


#!/usr/bin/env python
import hashlib
import os
import sys
from BaseHTTPServer import HTTPServer, BaseHTTPRequestHandler
from SocketServer import ThreadingMixIn
from ssl import wrap_socket, CERT_NONE
from socket import getfqdn
from urlparse import urlsplit
import re

from canari.maltego.utils import get_transform_version
from canari.maltego.message import (MaltegoTransformResponseMessage, MaltegoException, MaltegoTransformRequestMessage,
                                    MaltegoTransformExceptionMessage, MaltegoMessage)
from canari.pkgutils.transform import TransformDistribution
from common import fix_binpath, canari_main
from framework import SubCommand, Argument
from canari.config import config
import canari.resource


__author__ = 'Nadeem Douba'
__copyright__ = 'Copyright 2012, Canari Project'
__credits__ = []

__license__ = 'GPL'
__version__ = '0.8'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'


def message(m, response):
    """Write a MaltegoMessage to stdout and exit successfully"""

    response.send_response(200)
    response.send_header('Content-Type', 'text/xml')
    response.send_header('Connection', 'close')
    response.end_headers()

    v = None
    if isinstance(m, basestring):
        for url in re.findall("<iconurl>\s*(file://[^\s<]+)\s*</iconurl>(?im)", m):
            path = '/%s' % hashlib.md5(url).hexdigest()
            new_url = '%s://%s%s' % ('https' if response.server.is_ssl else 'http', response.server.hostname, path)
            if path not in response.server.resources:
                response.server.resources[path] = url[7:]
            m.replace(url, new_url, 1)
        v = m
    else:
        v = MaltegoMessage(m).render(fragment=True)
        # Get rid of those nasty unicode 32 characters
    response.wfile.write(v)


def croak(error_msg, response):
    """Throw an exception in the Maltego GUI containing error_msg."""

    response.send_response(200)
    response.send_header('Content-Type', 'text/xml')
    response.send_header('Connection', 'close')
    response.end_headers()

    response.wfile.write(
        MaltegoMessage(
            message=MaltegoTransformExceptionMessage(exceptions=[MaltegoException(error_msg)])
        ).render(fragment=True)
    )


class MaltegoTransformRequestHandler(BaseHTTPRequestHandler):
    protocol_version = 'HTTP/1.1'
    server_version = 'Canari/1.0'
    count = 0

    def dotransform(self, transform, valid_input_entity_types):
        try:
            if 'Content-Length' not in self.headers:
                self.send_error(500, 'What?')
                return
            request_str = self.rfile.read(int(self.headers['Content-Length']))

            msg = MaltegoTransformRequestMessage.parse(request_str).message

            e = msg.entity
            entity_type = e.type

            if valid_input_entity_types and entity_type not in valid_input_entity_types:
                self.send_error(400, 'Unsupported input entity!')
                return

            for k, i in msg.parameters.iteritems():
                if '.' in k:
                    config[k.replace('.', '/', 1)] = i
                else:
                    config['plume/%s' % k] = i

            msg = transform(
                msg,
                request_str if hasattr(transform, 'cmd') and
                callable(transform.cmd) else MaltegoTransformResponseMessage()
            ) if get_transform_version(transform) == 2 else transform(
                msg,
                request_str if hasattr(transform, 'cmd') and
                callable(transform.cmd) else MaltegoTransformResponseMessage(),
                config
            )

            if isinstance(msg, MaltegoTransformResponseMessage) or isinstance(msg, basestring):
                message(msg, self)
                return
            else:
                raise MaltegoException('Could not resolve message type returned by transform.')

        except MaltegoException, me:
            croak(str(me), self)
        except Exception, e:
            croak(str(e), self)

    def do_POST(self):
        path = urlsplit(self.path or '/').path
        if path not in self.server.transforms:
            self.send_error(404, "Duh?")
        else:
            self.dotransform(*self.server.transforms[path])

    def do_GET(self):
        path = urlsplit(self.path or '/').path
        if path in self.server.transforms:
            self.send_error(200, 'Yes')
            return
        elif path in self.server.resources:
            self.send_response(200)
            self.send_header('Content-Type', 'application/octet-stream')
            self.send_header('Connection', 'close')
            self.end_headers()
            self.wfile.write(file(self.server.resources[path]).read())
            return

        self.send_error(404, 'No')


class MaltegoHTTPServer(HTTPServer):
    server_name = 'Canari'
    resources = {}
    is_ssl = False

    def __init__(self, server_address=('', 8080), RequestHandlerClass=MaltegoTransformRequestHandler,
                 bind_and_activate=True, transforms={}, hostname=getfqdn()):
        HTTPServer.__init__(self, server_address, RequestHandlerClass, bind_and_activate)
        self.transforms = transforms
        self.hostname = hostname


class SecureMaltegoHTTPServer(MaltegoHTTPServer):
    is_ssl = True

    def __init__(self, server_address=('', 8080), RequestHandlerClass=MaltegoTransformRequestHandler,
                 bind_and_activate=True, transforms={}, cert='cert.pem', hostname=getfqdn()):
        MaltegoHTTPServer.__init__(
            self,
            server_address,
            RequestHandlerClass,
            bind_and_activate=bind_and_activate,
            transforms=transforms,
            hostname=hostname
        )
        self.socket = wrap_socket(self.socket, server_side=True, certfile=cert, cert_reqs=CERT_NONE)


class AsyncSecureMaltegoHTTPServer(ThreadingMixIn, SecureMaltegoHTTPServer):
    pass


class AsyncMaltegoHTTPServer(ThreadingMixIn, MaltegoHTTPServer):
    pass


def parse_args(args):
    if not args.hostname:
        args.hostname = getfqdn()
    return args


def monkey_patch(server):
    imgres = canari.resource.image_resource
    canari.resource.image_resource = lambda name, pkg=None: '%s://%s/%s' % (
        'https' if server.is_ssl else 'http',
        server.hostname,
        imgres(hashlib.md5(name).hexdigest())
    )
    canari.resource.icon_resource = canari.resource.image_resource
    callpkg = canari.resource.calling_package
    canari.resource.calling_package = lambda frame=4: callpkg(frame)


@SubCommand(
    canari_main,
    help='Runs a transform server for the given packages.',
    description='Runs a transform server for the given packages.'
)
@Argument(
    'packages',
    metavar='<package>',
    help='The name of the transform packages you wish to host (e.g. mypkg.transforms).',
    nargs='+'
)
@Argument(
    '--port',
    metavar='<port>',
    default=-1,
    type=int,
    help='The port the server will run on.'
)
@Argument(
    '--disable-ssl',
    action='store_true',
    default=False,
    help='Any extra parameters that can be sent to the local transform.'
)
@Argument(
    '--enable-privileged',
    action='store_true',
    default=False,
    help='DANGEROUS: permit TDS to run packages that require elevated privileges.'
)
@Argument(
    '--listen-on',
    metavar='[address]',
    default='',
    help='The address of the interface to listen on.'
)
@Argument(
    '--cert',
    metavar='[certificate]',
    default='cert.pem',
    help='The name of the certificate file used for the server in PEM format.'
)
@Argument(
    '--hostname',
    metavar='[hostname]',
    default=None,
    help='The hostname of this transform server.'
)
@Argument(
    '--daemon',
    default=False,
    action='store_true',
    help='Daemonize server (fork to background).'
)
def run_server(args):
    fix_binpath(config['default/path'])
    opts = parse_args(args)

    if opts.port == -1:
        opts.port = 443 if not opts.disable_ssl else 80

    if os.name == 'posix' and os.geteuid() and (opts.port <= 1024 or opts.enable_privileged):
        print ('You must run this server as root to continue...')
        os.execvp('sudo', ['sudo'] + sys.argv)

    transforms = {}

    print ('Loading transform packages...')

    try:
        for pkg_name in opts.packages:

            t = TransformDistribution(pkg_name)

            print ('Loading transform package %s' % pkg_name)
            for transform_module in t.remote_transforms:
                transform_name = transform_module.__name__

                if os.name == 'posix' and hasattr(transform_module.dotransform, 'privileged') and \
                        (os.geteuid() or not opts.enable_privileged):
                    continue

                if hasattr(transform_module.dotransform, 'remote') and transform_module.dotransform.remote:
                    print ('Loading %s at /%s...' % (transform_name, transform_name))
                    inputs = []
                    if hasattr(transform_module.dotransform, 'inputs'):
                        for category, entity_type in transform_module.dotransform.inputs:
                            inputs.append(entity_type._type_)
                            inputs.append(entity_type._v2type_)
                    transforms['/%s' % transform_name] = (transform_module.dotransform, inputs)

    except Exception, e:
        print (str(e))
        print ('Failed to load transforms... exiting')
        exit(-1)

    if not transforms:
        print ("Couldn't find any remote transforms... you sure you got this right?")
        exit(-1)

    httpd = None

    print ('Starting web server on %s:%s...' % (opts.listen_on, opts.port))
    server_address = (opts.listen_on, opts.port)

    if not opts.disable_ssl:
        if not os.path.exists(opts.cert):
            print ('The certificate file %s does not exist. Please create a PEM file...' % repr(opts.cert))
            exit(-1)
        print ('Making it secure (1337)...')
        httpd = AsyncSecureMaltegoHTTPServer(server_address=server_address,
                                             transforms=transforms, cert=opts.cert, hostname=opts.hostname)
    else:
        print ('Really? Over regular HTTP? What a shame...')
        httpd = AsyncMaltegoHTTPServer(server_address=server_address, transforms=transforms, hostname=opts.hostname)

    monkey_patch(httpd)

    if not opts.daemon or not os.fork():
        try:
            httpd.serve_forever()
        except KeyboardInterrupt:
            httpd.server_close()
    exit(0)



#!/usr/bin/env python

from common import canari_main
from canari.maltego.utils import highlight
from framework import SubCommand

__author__ = 'Nadeem Douba'
__copyright__ = 'Copyright 2012, Canari Project'
__credits__ = []

__license__ = 'GPL'
__version__ = '0.7'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'


@SubCommand(
    canari_main,
    help='Lists all the available canari commands.',
    description='Lists all the available canari commands.'
)
def list_commands(opts):
    cmds = canari_main.subparsers.choices
    k = cmds.keys()
    k.sort()
    for i in k:
        print ('%s - %s' % (highlight(i, 'green', True), cmds[i].description))


#!/usr/bin/env python

import os
import imp
import re
from canari.maltego.configuration import MaltegoEntity
from canari.pkgutils.maltego import MaltegoDistribution, MtzDistribution

from common import canari_main, project_tree, parse_bool
from framework import SubCommand, Argument
from canari.maltego.entities import Entity


__author__ = 'Nadeem Douba'
__copyright__ = 'Copyright 2012, Canari Project'
__credits__ = ['Nadeem Douba']

__license__ = 'GPL'
__version__ = '0.5'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'


def parse_args(args):
    if args.outfile is None:
        try:
            args.outfile = os.path.join(project_tree()['transforms'], 'common', 'entities.py')
        except ValueError:
            args.outfile = 'entities.py'
    if args.maltego_entities:
        args.namespace.extend(args.exclude_namespace)
        args.exclude_namespace = []
    return args


def normalize_fn(fn):
    # Get rid of starting underscores or numbers and bad chars for var names in python
    return re.sub(r'[^A-Za-z0-9]', '', re.sub(r'^[^A-Za-z]+', '', fn))


def get_existing_entities(filename):
    m = imp.load_source('entities', filename)
    l = []
    for c in dir(m):
        try:
            entity_class = m.__dict__[c]
            if issubclass(entity_class, Entity):
                l.append(entity_class)
        except TypeError:
            pass
    return l


class DirFile(object):
    def __init__(self, path):
        self.path = path

    def namelist(self):
        l = []
        for base, dirs, files in os.walk(self.path):
            l.extend([os.path.join(base, f) for f in files])
        return l

    def open(self, fname):
        return file(fname)


@SubCommand(
    canari_main,
    help='Converts Maltego entity definition files to Canari python classes. Excludes Maltego built-in entities.',
    description='Converts Maltego entity definition files to Canari python classes. '
                 'Excludes Maltego built-in entities.'
)
@Argument(
    'outfile',
    metavar='[output file]',
    help='Which file to write the output to.',
    default=None,
    nargs='?'
)
@Argument(
    '--mtz-file',
    '-m',
    metavar='<mtzfile>',
    help='A *.mtz file containing an export of Maltego entities.',
    required=False
)
@Argument(
    '--exclude-namespace',
    '-e',
    metavar='<namespace>',
    help='Name of Maltego entity namespace to ignore. Can be defined multiple times.',
    required=False,
    action='append',
    default=['maltego', 'maltego.affiliation']
)
@Argument(
    '--namespace',
    '-n',
    metavar='<namespace>',
    help='Name of Maltego entity namespace to generate entity classes for. Can be defined multiple times.',
    required=False,
    action='append',
    default=[]
)
@Argument(
    '--maltego-entities',
    '-M',
    help="Generate entities belonging to the 'maltego' namespace.",
    default=False,
    action='store_true'
)
@Argument(
    '--append',
    '-a',
    help='Whether or not to append to the existing *.py file.',
    action='store_true',
    default=False
)
@Argument(
    '--entity',
    '-E',
    metavar='<entity>',
    help='Name of Maltego entity to generate Canari python class for.',
    required=False,
    action='append',
    default=[]
)
def generate_entities(args):
    opts = parse_args(args)

    if os.path.exists(opts.outfile) and not opts.append and not \
        parse_bool('%s already exists. Are you sure you want to overwrite it?' % repr(opts.outfile),
                   default='n'):
        exit(-1)

    distribution = None
    if not opts.mtz_file:
        distribution = MaltegoDistribution()
        if distribution.version >= '3.4.0':
            print("""
=========================== ERROR: NOT SUPPORTED ===========================

 Starting from Maltego v3.4.0 the 'canari generate-entities' command can no
 longer generate entity definition files from the Maltego configuration
 directory. Entities can only be generated from export files (*.mtz). To
 export entities navigate to the 'Manage' tab in Maltego, then click on the
 'Export Entities' button and follow the prompts. Once the entities have
 been exported, run the following command:

 shell> canari generate-entities -m myentities.mtz

=========================== ERROR: NOT SUPPORTED ===========================
                """)
            exit(-1)
    else:
        distribution = MtzDistribution(opts.mtz_file)

    namespaces = dict()

    excluded_entities = []
    if opts.append:
        existing_entities = get_existing_entities(opts.outfile)
        # excluded_entities.extend([e._type_ for e in existing_entities])
        for entity_class in existing_entities:
            excluded_entities.extend(entity_class._type_)
            if entity_class._type_.endswith('Entity'):
                namespaces[entity_class._namespace_] = entity_class.__name__

    print 'Generating %s...' % repr(opts.outfile)
    outfile = open(opts.outfile, 'ab' if opts.append else 'wb')

    if opts.append:
        outfile.write('\n\n')
    else:
        outfile.write('#!/usr/bin/env python\n\nfrom canari.maltego.entities import EntityField, Entity\n\n\n')

    for entity_file in distribution.entity_files:
        entity = MaltegoEntity.parse(
            distribution.read_file(entity_file)
        )

        if (opts.entity and entity.id not in opts.entity) or entity.id in excluded_entities:
            continue

        namespace_entity = entity.id.split('.')

        base_classname = None
        namespace = '.'.join(namespace_entity[:-1])
        name = namespace_entity[-1]
        classname = name

        if (opts.namespace and namespace not in opts.namespace) or namespace in opts.exclude_namespace:
            continue

        print 'Generating entity definition for %s...' % entity_file
        if namespace not in namespaces:
            base_classname = '%sEntity' % (''.join([n.title() for n in namespace_entity[:-1]]))
            namespaces[namespace] = base_classname
            outfile.write('class %s(Entity):\n    _namespace_ = %s\n\n' % (base_classname, repr(namespace)))
        else:
            base_classname = namespaces[namespace]

        for field in entity.properties.fields.itervalues():
            fields = [
                'name=%s' % repr(field.name),
                'propname=%s' % repr(normalize_fn(field.name)),
                'displayname=%s' % repr(field.displayname)

            ]
            outfile.write('@EntityField(%s)\n' % ', '.join(fields))

        outfile.write('class %s(%s):\n    pass\n\n\n' % (classname, base_classname))

    outfile.close()
    print 'done.'


#!/usr/bin/env python

import inspect
import signal
import sys
import os
import traceback
from types import ModuleType
from xml.etree.cElementTree import fromstring
from safedexml import Model

from canari.commands.common import sudo, import_transform
from canari.maltego.entities import Unknown
from message import MaltegoMessage, MaltegoTransformExceptionMessage, MaltegoException, \
    MaltegoTransformResponseMessage, MaltegoTransformRequestMessage, UIMessage, Field

__author__ = 'Nadeem Douba'
__copyright__ = 'Copyright 2012, Canari Project'
__credits__ = []

__license__ = 'GPL'
__version__ = '0.2'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'

__all__ = [
    'onterminate',
    'message',
    'highlight',
    'console_message',
    'croak',
    'guess_entity_type',
    'to_entity',
    'get_transform_version',
    'local_transform_runner',
    'debug',
    'progress'
]


def onterminate(func):
    """Register a signal handler to execute when Maltego forcibly terminates the transform."""
    signal.signal(signal.SIGTERM, func)
    signal.signal(signal.SIGINT, func)


def message(m, fd=sys.stdout):
    """Write a MaltegoMessage to stdout and exit successfully"""
    print MaltegoMessage(message=m).render(fragment=True)
    sys.exit(0)


def highlight(s, color, bold):
    """
    Internal API: Returns the colorized version of the text to be returned to a POSIX terminal. Not compatible with
    Windows (yet).
    """
    if os.name == 'posix':
        attr = []
        if color == 'green':
            # green
            attr.append('32')
        elif color == 'red':
            # red
            attr.append('31')
        else:
            attr.append('30')
        if bold:
            attr.append('1')
        s = '\x1b[%sm%s\x1b[0m' % (';'.join(attr), s)

    return s


def console_message(msg, tab=-1):
    """
    Internal API: Returns a prettified tree-based output of an XML message for debugging purposes. This helper function
    is used by the debug-transform command.
    """
    tab += 1

    if isinstance(msg, Model):
        msg = fromstring(msg.render(fragment=True))

    print('%s`- %s: %s %s' % (
        '  ' * tab,
        highlight(msg.tag, None, True),
        highlight(msg.text, 'red', False) if msg.text is not None else '',
        highlight(msg.attrib, 'green', True) if msg.attrib.keys() else ''
    ))
    for c in msg.getchildren():
        print('  %s`- %s: %s %s' % (
            '  ' * tab,
            highlight(c.tag, None, True),
            highlight(c.text, 'red', False) if c.text is not None else '',
            highlight(c.attrib, 'green', True) if c.attrib.keys() else ''
        ))
        for sc in c.getchildren():
            tab += 1
            console_message(sc, tab)
            tab -= 1


def croak(error_msg, message_writer=message):
    """Throw an exception in the Maltego GUI containing error_msg."""
    message_writer(MaltegoTransformExceptionMessage(exceptions=[MaltegoException(error_msg)]))


def guess_entity_type(transform_module, fields):
    """
    Internal API: Returns the entity type based on the following best match algorithm:

    1. If a transform does not specify the input entity types, the Unknown entity will be returned.
    2. If a transform only has one input entity type, then that entity type will be returned.
    3. If a transform has more than one input entity type, then the entity type that has the
       most number of matching entity fields in the entity's class definition will be returned.

    This is only used by the local transform runner to detect the input entity type since this information is excluded
    at run-time.
    """
    if not hasattr(transform_module.dotransform, 'inputs') or not transform_module.dotransform.inputs:
        return Unknown
    if len(transform_module.dotransform.inputs) == 1 or not fields:
        return transform_module.dotransform.inputs[0][1]
    num_matches = 0
    best_match = Unknown
    for category, entity_type in transform_module.dotransform.inputs:
        l = len(set(entity_type._fields_to_properties_.keys()).intersection(fields.keys()))
        if l > num_matches:
            num_matches = l
            best_match = entity_type
    return best_match


def to_entity(entity_type, value, fields):
    """
    Internal API: Returns an instance of an entity of type entity_type with the specified value and fields (stored in
    dict). This is only used by the local transform runner as a helper function.
    """
    e = entity_type(value)
    for k, v in fields.iteritems():
        e.fields[k] = Field(k, v)
    return e


def get_transform_version(transform):
    """
    Internal API: Returns the version of the transform function based on the transform function's signature. Currently,
    only two versions are supported (2 and 3). This is what version 2 transform functions look like:

    def transform(request, response):
        ...

    Version 3 transforms have the additional config variable like so:

    def transform(request, response, config):
        ...

    Or can have a varargs parameter as a third argument:

    def transform(request, response, *args):
        ...

    In both cases, version 3 transforms will be passed a local copy of the canari configuration object as the third
    argument. However, in the latter example, the configuration object will be stored in a tuple (i.e. (config,)).
    """
    spec = inspect.getargspec(transform)
    if spec.varargs is not None:
        return 3
    n = len(spec.args)
    if 2 <= n <= 3:
        return n
    raise Exception('Could not determine transform version.')


def local_transform_runner(transform, value, fields, params, config, message_writer=message):
    """
    Internal API: The local transform runner is responsible for executing the local transform.

    Parameters:

    transform      - The name or module of the transform to execute (i.e sploitego.transforms.whatismyip).
    value          - The input entity value.
    fields         - A dict of the field names and their respective values.
    params         - The extra parameters passed into the transform via the command line.
    config         - The Canari configuration object.
    message_writer - The message writing function used to write the MaltegoTransformResponseMessage to stdout. This is
                     can either be the console_message or message functions. Alternatively, the message_writer function
                     can be any callable object that accepts the MaltegoTransformResponseMessage as the first parameter
                     and writes the output to a destination of your choosing.

    This helper function is only used by the run-transform, debug-transform, and dispatcher commands.
    """
    transform_module = None
    try:
        transform_module = transform if isinstance(transform, ModuleType) else import_transform(transform)

        if os.name == 'posix' and hasattr(transform_module.dotransform, 'privileged') and os.geteuid():
            rc = sudo(sys.argv)
            if rc == 1:
                message_writer(MaltegoTransformResponseMessage() + UIMessage('User cancelled transform.'))
            elif rc == 2:
                message_writer(MaltegoTransformResponseMessage() + UIMessage('Too many incorrect password attempts.'))
            elif rc:
                message_writer(MaltegoTransformResponseMessage() + UIMessage('Unknown error occurred.'))
            sys.exit(rc)

        if hasattr(transform_module, 'onterminate'):
            onterminate(transform_module.onterminate)
        else:
            transform_module.__setattr__('onterminate', lambda *args: sys.exit(-1))

        input_entity = to_entity(guess_entity_type(transform_module, fields), value, fields)

        msg = transform_module.dotransform(
            MaltegoTransformRequestMessage(
                entities=[input_entity.__entity__],
                parameters={'canari.local.arguments': Field(name='canari.local.arguments', value=params)}
            ),
            MaltegoTransformResponseMessage()
        ) if get_transform_version(transform_module.dotransform) == 2 else transform_module.dotransform(
            MaltegoTransformRequestMessage(
                entities=[input_entity.__entity__],
                parameters={'canari.local.arguments': Field(name='canari.local.arguments', value=params)}
            ),
            MaltegoTransformResponseMessage(),
            config
        )
        if isinstance(msg, MaltegoTransformResponseMessage):
            message_writer(msg)
        elif isinstance(msg, basestring):
            raise MaltegoException(msg)
        else:
            raise MaltegoException('Could not resolve message type returned by transform.')
    except MaltegoException, me:
        croak(str(me), message_writer)
    except ImportError:
        e = traceback.format_exc()
        croak(e, message_writer)
    except Exception:
        e = traceback.format_exc()
        croak(e, message_writer)
    except KeyboardInterrupt:
        if transform_module:
            transform_module.onterminate()


def debug(*args):
    """Send debug messages to the Maltego console."""
    for i in args:
        sys.stderr.write('D:%s\n' % str(i))
        sys.stderr.flush()


def progress(i):
    """Send a progress report to the Maltego console."""
    sys.stderr.write('%%%d\n' % min(max(i, 0), 100))
    sys.stderr.flush()


#!/usr/bin/env python

import os
from canari.pkgutils.transform import TransformDistribution

from common import canari_main, parse_bool
from framework import SubCommand, Argument


__author__ = 'Nadeem Douba'
__copyright__ = 'Copyright 2012, Canari Project'
__credits__ = []

__license__ = 'GPL'
__version__ = '0.6'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'


# Dictionary of detected transforms
transforms = {}


# Main
@SubCommand(
    canari_main,
    help="Loads a canari package into Plume.",
    description="Loads a canari package into Plume."
)
@Argument(
    'package',
    metavar='<package>',
    help='the name of the canari transforms package to load into Plume.'
)
@Argument(
    '-d',
    '--plume-dir',
    metavar='[www dir]',
    default=os.getcwd(),
    help='the path where Plume is installed.'
)
def load_plume_package(opts):

    if not os.path.exists(os.path.join(opts.plume_dir, 'plume.py')):
        print('Plume does not appear to be installed in %s.' % opts.plume_dir)
        ans = parse_bool("Would you like to install it [Y/n]: ")
        if not ans:
            print 'Installation cancelled. Quitting...'
            exit(-1)
        os.system('canari install-plume --install-dir %s' % opts.plume_dir)
        opts.plume_dir = os.path.join(opts.plume_dir, 'plume')

    transform_package = None
    try:
        transform_package = TransformDistribution(opts.package)
    except ValueError:
        exit(-1)

    if transform_package.has_remote_transforms:
        try:
            transform_package.configure(opts.plume_dir, remote=True)
        except ImportError:
            pass
        print('Please restart plume (./plume.sh restart) for changes to take effect.')
        exit(0)

    print ('Error: no remote transforms found. '
           'Please make sure that at least one transform has remote=True in @configure before retrying.')
    exit(-1)



#!/usr/bin/env python

import os

from tempfile import NamedTemporaryFile, gettempdir
from time import time


if os.name == 'nt':
    import msvcrt
    from ctypes import *
    from ctypes.wintypes import BOOL, DWORD, HANDLE

    LOCK_SH = 0x0  # the default
    LOCK_NB = 0x1  # LOCKFILE_FAIL_IMMEDIATELY
    LOCK_EX = 0x2  # LOCKFILE_EXCLUSIVE_LOCK
    LOCK_UN = 0x4  # Unlock file. Not in NT API, just needs to be there.

    # --- the code is taken from pyserial project ---
    #
    # detect size of ULONG_PTR
    def is_64bit():
        return sizeof(c_ulong) != sizeof(c_void_p)

    if is_64bit():
        ULONG_PTR = c_int64
    else:
        ULONG_PTR = c_ulong
    PVOID = c_void_p

    # --- Union inside Structure by stackoverflow:3480240 ---
    class _OFFSET(Structure):
        _fields_ = [
            ('Offset', DWORD),
            ('OffsetHigh', DWORD)]

    class _OFFSET_UNION(Union):
        _anonymous_ = ['_offset']
        _fields_ = [
            ('_offset', _OFFSET),
            ('Pointer', PVOID)]

    class OVERLAPPED(Structure):
        _anonymous_ = ['_offset_union']
        _fields_ = [
            ('Internal', ULONG_PTR),
            ('InternalHigh', ULONG_PTR),
            ('_offset_union', _OFFSET_UNION),
            ('hEvent', HANDLE)]

    LPOVERLAPPED = POINTER(OVERLAPPED)

    # --- Define function prototypes for extra safety ---
    LockFileEx = windll.kernel32.LockFileEx
    LockFileEx.restype = BOOL
    LockFileEx.argtypes = [HANDLE, DWORD, DWORD, DWORD, DWORD, LPOVERLAPPED]
    UnlockFileEx = windll.kernel32.UnlockFileEx
    UnlockFileEx.restype = BOOL
    UnlockFileEx.argtypes = [HANDLE, DWORD, DWORD, DWORD, LPOVERLAPPED]

    def flock(file_, flags):
        hfile = msvcrt.get_osfhandle(file_.fileno())
        overlapped = OVERLAPPED()
        if flags & LOCK_UN and UnlockFileEx(hfile, 0, 0, 0xFFFF0000, byref(overlapped)):
            return
        elif (not flags or flags & (LOCK_EX | LOCK_NB | LOCK_SH)) and \
                LockFileEx(hfile, flags, 0, 0, 0xFFFF0000, byref(overlapped)):
            return
        raise IOError(GetLastError())

else:
    from fcntl import flock, LOCK_EX, LOCK_NB, LOCK_SH, LOCK_UN


__author__ = 'Nadeem Douba'
__copyright__ = 'Copyright 2012, Canari Project'
__credits__ = ['Jonathan Feinberg']

__license__ = 'GPL'
__version__ = '0.3'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'

__all__ = [
    'cookie',
    'flock',
    'fsemaphore',
    'fmutex',
    'ufile',
    'age'
]


def cookie(name):
    return os.path.join(gettempdir(), name)


class fsemaphore(file):

    def __init__(self, name, mode='rb', buffering=-1):
        super(fsemaphore, self).__init__(name, mode, buffering)

    def lockex(self, nb=False):
        flags = LOCK_EX
        if nb:
            flags |= LOCK_NB
        flock(self, flags)

    def locksh(self, nb=False):
        flags = LOCK_SH
        if nb:
            flags |= LOCK_NB
        flock(self, flags)

    def unlock(self, nb=False):
        flags = LOCK_UN
        if nb:
            flags |= LOCK_NB
        flock(self, flags)


class fmutex(fsemaphore):

    def __init__(self, name):
        super(fmutex, self).__init__(cookie(name), 'wb')
        self.lockex()

    def __del__(self):
        self.unlock()


def ufile(name, delete=False):
    n, e = os.path.splitext(name)
    f = NamedTemporaryFile(suffix=e, prefix='%s_' % n, delete=delete)
    if os.name == 'posix':
        os.chmod(f.name, 0644)
    return f


def age(path):
    return time() - os.stat(path).st_mtime



#!/usr/bin/env python

import os

from canari.pkgutils.transform import TransformDistribution
from common import canari_main, uproot, pushd
from framework import SubCommand, Argument


__author__ = 'Nadeem Douba'
__copyright__ = 'Copyright 2012, Canari Project'
__credits__ = []

__license__ = 'GPL'
__version__ = '0.3'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'


# Extra sauce to parse args
def parse_args(args):
    uproot()
    return args


# Main
# Argument parser
@SubCommand(
    canari_main,
    help='Creates an importable Maltego profile (*.mtz) file.',
    description='Creates an importable Maltego profile (*.mtz) file.'
)
@Argument(
    'package',
    metavar='<package>',
    help='the name of the canari transforms package to install.'
)
@Argument(
    '-w',
    '--working-dir',
    metavar='[working dir]',
    default=None,
    help='the path that will be used as the working directory for the '
         'transforms being installed (default: current working directory)'
)
def create_profile(args):

    opts = parse_args(args)
    current_dir = os.getcwd()
    try:
        with pushd(opts.working_dir or current_dir):
            transform_package = TransformDistribution(opts.package)
            transform_package.create_profile(opts.working_dir, current_dir)
    except ValueError, e:
        print(str(e))
        exit(-1)


#!/usr/bin/env python

from framework import SubCommand, Argument
from common import canari_main


__author__ = 'Nadeem Douba'
__copyright__ = 'Copyright 2012, Canari Project'
__credits__ = []

__license__ = 'GPL'
__version__ = '0.3'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'


@SubCommand(
    canari_main,
    help='Shows help related to various canari commands',
    description='Shows help related to various canari commands'
)
@Argument(
    'command',
    metavar='<command>',
    choices=canari_main.subparsers.choices,
    default='help',
    nargs='?',
    help='The canari command you want help for.'
)
def help(opts):
    canari_main.subparsers.choices[opts.command].print_help()


#!/usr/bin/env python

from inspect import stack, getmodule

__author__ = 'Nadeem Douba'
__copyright__ = 'Copyright 2012, Canari Project'
__credits__ = []

__license__ = 'GPL'
__version__ = '0.1'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'

__all__ = [
    'calling_module',
    'calling_package'
]


def calling_module(frame=2):
    frame = stack()[frame]
    return getmodule(frame[0])


def calling_package(frame=2):
    frame = stack()[frame]
    return getmodule(frame[0]).__name__.split('.')[0]


__author__ = 'Nadeem Douba'
__copyright__ = 'Copyright 2012, Canari Project'
__credits__ = []

__license__ = 'GPL'
__version__ = '1.1'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'

__all__ = [
    'commands',
    'maltego',
    'pkgutils',
    'resources',
    'tds',
    'utils',
    'xmltools',
    'config',
    'easygui',
    'framework',
    'resource'
]


#!/usr/bin/env python

import os
import re

from common import write_template, read_template, canari_main, init_pkg, project_tree
from framework import SubCommand, Argument

__author__ = 'Nadeem Douba'
__copyright__ = 'Copyright 2012, Canari Project'
__credits__ = []

__license__ = 'GPL'
__version__ = '0.4'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'


def parse_args(args):
    if args.transform_dir is None:
        args.transform_dir = project_tree()['transforms']
    if args.transform in ['common', 'common.py']:
        print "Error: 'common' is a reserved module. Please name your transform something else."
        exit(-1)
    return args


@SubCommand(
    canari_main,
    help='Creates a new transform in the specified directory and auto-updates __init__.py.',
    description='Creates a new transform in the specified directory and auto-updates __init__.py.'
)
@Argument(
    'transform',
    metavar='<transform name>',
    help='The name of the transform you wish to create.'
)
@Argument(
    '-d',
    '--transform-dir',
    metavar='<dir>',
    help='The directory in which you wish to create the transform.',
    default=None
)
def create_transform(args):

    opts = parse_args(args)

    initf = os.path.join(opts.transform_dir, '__init__.py')
    transform = opts.transform if not opts.transform.endswith('.py') else opts.transform[:-3]

    if '.' in transform:
        print "Transform name (%s) cannot have a dot ('.')." % repr(transform)
        exit(-1)
    elif not transform:
        print "You must specify a valid transform name."
        exit(-1)

    directory = opts.transform_dir
    transformf = os.path.join(directory, opts.transform if opts.transform.endswith('.py') else '%s.py' % opts.transform )

    if not os.path.exists(initf):
        print ('Directory %s does not appear to be a python package directory... quitting!' % repr(opts.transform_dir))
        exit(-1)
    if os.path.exists(transformf):
        print ('Transform %s already exists... quitting' % repr(transformf))
        exit(-1)

    values = init_pkg()

    write_template(
        transformf,
        read_template('newtransform', values)
    )

    print ('updating %s' % initf)
    init = file(initf).read()

    with file(initf, mode='wb') as w:
        w.write(
            re.sub(
                r'__all__\s*\=\s*\[',
                '__all__ = [\n    %s,' % repr(transform),
                init
            )
        )

    print ('done!')



#!/usr/bin/env python

import os
import sys
import string

from pkg_resources import resource_listdir, resource_filename

from canari.config import CanariConfigParser
from canari.pkgutils.maltego import MaltegoDistribution, MtzDistribution
from canari.commands.common import import_package, import_transform, parse_bool


__author__ = 'Nadeem Douba'
__copyright__ = 'Copyright 2012, Canari Project'
__credits__ = ['Nadeem Douba']

__license__ = 'GPL'
__version__ = '0.1'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'

__all__ = [
    'TransformDistribution'
]


class TransformDistribution(object):

    def __init__(self, package_name):
        self._package_name = package_name.replace('.transforms', '') \
            if package_name.endswith('.transforms') else package_name

        print('Looking for transforms in %s.transforms...' % self.name)
        try:
            self._package = import_package('%s.transforms' % self.name)
        except ImportError, e:
            raise ValueError("Does not appear to be a valid canari package. "
                             "Couldn't import the '%s.transforms' package in '%s'. Error message: %s" %
                             (self.name, self.name, e))

        self._transforms = []
        self._remote_transforms = []
        self._load_transforms()
        self._config = '%s.conf' % self.name
        self._resources = '%s.resources' % self.name
        self._package_path = os.path.abspath(os.path.dirname(self._package.__file__)).replace('/transforms', '')
        self._default_prefix = os.path.join(os.path.expanduser('~'), '.canari') if self.is_site_package else os.getcwd()
        self._machines = [
            m for m in resource_listdir(self.get_resource_module('maltego'), '') if m.endswith('.machine')
        ]
        if not self.has_transforms:
            raise ValueError('Error: no transforms found...')
        print('Package loaded.')

    def _load_transforms(self):
        for t in self._package.__all__:
            m = import_transform('%s.transforms.%s' % (self.name, t))
            if hasattr(m, 'dotransform') and hasattr(m.dotransform, 'label'):
                self._transforms.append(m)
                if hasattr(m.dotransform, 'remote') and m.dotransform.remote:
                    self._remote_transforms.append(m)

    @property
    def name(self):
        return self._package_name

    @property
    def package_path(self):
        return self._package_path

    @property
    def transforms(self):
        return self._transforms

    @property
    def remote_transforms(self):
        return self._remote_transforms

    @property
    def has_remote_transforms(self):
        return bool(self.remote_transforms)

    @property
    def has_transforms(self):
        return bool(self.transforms)

    @property
    def config_file(self):
        return self._config

    @property
    def resources(self):
        return self._resources

    @property
    def is_site_package(self):
        for p in sys.path:
            if p and self.package_path.startswith(p):
                return True
        return False

    @property
    def machines(self):
        return self._machines

    @property
    def entities_file(self):
        return resource_filename(self.get_resource_module('maltego'), 'entities.mtz')

    @property
    def default_prefix(self):
        return self._default_prefix

    def get_resource_module(self, name):
        return '%s.%s' % (self._resources, name)

    @staticmethod
    def _write_config(src, dst, **kwargs):
        if os.path.lexists(dst) and not \
                parse_bool('%s already exists. Would you like to overwrite it?' % dst, default=False):
            return

        print ('Writing %s to %s' % (src, dst))
        with file(dst, mode='wb') as w:
            if kwargs.pop('is_template', None):
                w.write(
                    string.Template(
                        file(
                            src
                        ).read()
                    ).substitute(**kwargs)
                )
            else:
                w.write(
                    file(
                        src
                    ).read()
                )

    def _update_config(self, canari_config, load=True, remote=False):
        ld = os.getcwd()
        os.chdir(os.path.dirname(canari_config))

        config = CanariConfigParser()
        config.read(canari_config)
        configs = config['default/configs']
        packages = config['remote/packages']

        if isinstance(configs, basestring):
            configs = [configs] if configs else []

        if isinstance(packages, basestring):
            packages = [packages] if packages else []

        if load:
            if self.config_file not in configs:
                print ('Updating %s...' % canari_config)
                configs.append(self.config_file)
                config['default/configs'] = ','.join(configs)

            if self.name not in packages and remote:
                packages.append(self.name)
                config['remote/packages'] = ','.join(packages)
        else:
            if self.config_file in configs:
                print ('Updating %s...' % canari_config)
                configs.remove(self.config_file)
                config['default/configs'] = ','.join(configs)

            if self.name in packages and remote:
                packages.remove(self.name)
                config['remote/packages'] = ','.join(packages)

        config.write(file(canari_config, mode='wb'))
        os.chdir(ld)

    def configure(self, install_prefix, load=True, remote=False, **kwargs):
        if load:
            canari_config = resource_filename('canari.resources.template', 'canari.plate')
            self._write_config(
                canari_config,
                os.path.join(install_prefix, 'canari.conf'),
                is_template=True,
                command=' '.join(sys.argv),
                config=self.config_file if self.name != 'canari' else '',
                path='${PATH},/usr/local/bin,/opt/local/bin' if os.name == 'posix' else ''
            )

        if self._package_name != 'canari':
            if load:
                package_config = resource_filename(self.get_resource_module('etc'), self.config_file)
                self._write_config(package_config, os.path.join(install_prefix, self.config_file))
            self._update_config(os.path.join(install_prefix, 'canari.conf'), load, remote)

    def _init_install_prefix(self, install_prefix):
        if not install_prefix:
            install_prefix = self.default_prefix
        if not os.path.lexists(install_prefix):
            os.makedirs(install_prefix)
        return install_prefix

    def install(self, install_prefix, distribution, configure=True, remote=False):
        if isinstance(distribution, basestring) or not distribution:
            distribution = MaltegoDistribution(distribution)
        if not isinstance(distribution, MtzDistribution) and distribution.version >= '3.4.0':
            raise ValueError("""
!!!!!!!!!!!!!!!!!!!!!!!!!!! ERROR: NOT SUPPORTED !!!!!!!!!!!!!!!!!!!!!!!!!!!

 Starting from Maltego Radium (v3.4.0) the 'canari install-package' command
 is no longer supported. Please use the 'canari create-profile' command,
 instead. This will create an importable config file (*.mtz) which can be
 imported using the 'Import Configuration' option in Maltego. This option
 can be found by clicking on the <Maltego icon> in the top left corner of
 your Maltego window then scrolling to 'Import' then 'Import Configuration'.

 NOTE: This command will automatically install and configure the
 'canari.conf' file for you in the default location for your OS.

 EXAMPLE:

 shell> canari create-profile sploitego
 ...
 shell> ls
 sploitego.mtz <--- Import this file

!!!!!!!!!!!!!!!!!!!!!!!!!!! ERROR: NOT SUPPORTED !!!!!!!!!!!!!!!!!!!!!!!!!!!
            """)

        if not isinstance(distribution, MtzDistribution):
            print 'Installing transform package %s...' % self.name

        install_prefix = self._init_install_prefix(install_prefix)

        self._install_transforms(install_prefix, distribution)
        self._install_entities(distribution)
        self._install_machines(distribution)

        if configure:
            self.configure(install_prefix, remote=remote)

    def _install_transforms(self, prefix, distribution):
        for transform in self.transforms:
            distribution.add_transform(prefix, 'Local', transform, server='Local')

    def _install_entities(self, distribution):
        try:
            src = self.entities_file
            if not os.path.lexists(src):
                return
            mtz = MtzDistribution(src)

            for entity_file in mtz.entity_files:
                distribution.add_entity(mtz.read_file(entity_file))

            for icon_category in mtz.icon_categories:
                icon_category_dir = distribution.get_icon_category_dir(icon_category)
                for icon in mtz.get_icons_by_category(icon_category):
                    data = mtz.read_file(icon)
                    p = distribution.path_join(icon_category_dir, os.path.basename(icon))
                    print 'Installing custom icon %s to %s...' % (icon, p)
                    distribution.write_file(p, data)

        except ImportError:
            pass

    def _install_machines(self, distribution):
        try:
            package = self.get_resource_module('maltego')
            for machine in self.machines:
                src = resource_filename(package, machine)
                distribution.add_machine(machine, open(src).read())
        except ImportError:
            pass

    def uninstall(self, install_prefix, maltego_prefix=None):
        distribution = MaltegoDistribution(maltego_prefix)
        if distribution.version >= '3.4.0':
            raise ValueError("""
!!!!!!!!!!!!!!!!!!!!!!!!!!! ERROR: NOT SUPPORTED !!!!!!!!!!!!!!!!!!!!!!!!!!!

 Starting from Maltego Radium (v3.4.0) the 'canari uninstall-package'
 command is no longer supported. Please use the Maltego interface to
 uninstall packages.

!!!!!!!!!!!!!!!!!!!!!!!!!!! ERROR: NOT SUPPORTED !!!!!!!!!!!!!!!!!!!!!!!!!!!
            """)

        install_prefix = self._init_install_prefix(install_prefix)

        self._uninstall_transforms(distribution)
        self._uninstall_machines(distribution)
        self.configure(install_prefix, load=False)

    def _uninstall_machines(self, distribution):
        try:
            for machine in self.machines:
                distribution.remove_machine(machine)
        except ImportError:
            pass

    def _uninstall_transforms(self, distribution):
        for transform in self.transforms:
            distribution.remove_transform('Local', transform, server='Local')

    def create_profile(self, install_prefix, current_dir, configure=True):
        mtz = os.path.join(current_dir, '%s.mtz' % self.name)
        print('Creating profile %s...' % mtz)
        mtz = MtzDistribution(mtz, 'w')
        self.install(install_prefix, mtz, configure)
        mtz.close()

        print("""
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% SUCCESS! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

 Successfully created %s. You may now import this file into
 Maltego.

 INSTRUCTIONS:
 -------------
 1. Open Maltego.
 2. Click on the home button (Maltego icon, top-left corner).
 3. Click on 'Import'.
 4. Click on 'Import Configuration'.
 5. Follow prompts.
 6. Enjoy!

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% SUCCESS! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            """ % mtz.filename)


#!/usr/bin/env python

from resource import conf

from re import findall, search, match, split
from ConfigParser import SafeConfigParser
from os import environ, getcwd, path
from utils.wordlist import wordlist
from urlparse import urlparse


__author__ = 'Nadeem Douba'
__copyright__ = 'Copyright 2012, Canari Project'
__credits__ = []

__license__ = 'GPL'
__version__ = '0.3'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'

__all__ = [
    'CanariConfigParser',
    'config'
]


class CanariConfigParser(SafeConfigParser):
    def _interpolate_environment_variables(self, value):
        if isinstance(value, str):
            evs = self._get_env_vars(value)
            if evs:
                for ev in evs:
                    s = '${%s}' % ev
                    value = value.replace(s, environ[ev])
        return value

    def _interpolate(self, section, option, value, d):
        if isinstance(value, str):
            value = self._interpolate_environment_variables(value)
        elif isinstance(value, dict):
            for i in value:
                value[i] = self._interpolate_environment_variables(value[i])
        else:
            for i in range(0, len(value)):
                value[i] = self._interpolate_environment_variables(value[i])
        value = SafeConfigParser._interpolate(self, section, option, value, d)
        return value

    def _get_env_vars(self, value):
        return findall(r'\${(.+?)}', value)

    def __iadd__(self, other):
        self.add_section(other)
        return self

    def __isub__(self, other):
        self.remove_section(other)
        return self

    def __getitem__(self, item):
        section, option = item.split('/')
        value = self.get(section, option)
        if isinstance(value, basestring):
            if value.startswith('module:'):
                r = urlparse(value)
                try:
                    v = r.path.lstrip('/')
                    m = __import__(r.netloc, globals(), locals(), [v])
                    value = m.__dict__[v]
                except ImportError:
                    pass
            elif match(r'^\d+$', value) is not None:
                value = int(value)
            elif match(r'^\d+\.\d+$', value) is not None:
                value = float(value)
            elif search(r'\s*(?<=[^\\]),+\s*', value) is not None:
                l = split(r'\s*(?<=[^\\]),+\s*', value)
                value = []
                for v in l:
                    if match(r'^\d+$', v) is not None:
                        v = int(v)
                    elif match(r'^\d+\.\d+$', v) is not None:
                        v = float(v)
                    else:
                        v = v.replace(r'\,', ',')
                    value.append(v)
            else:
                value = value.replace(r'\,', ',')
        if option == 'wordlist':
            value = wordlist(value)
        return value

    def __setitem__(self, key, value):
        section, option = key.split('/', 1)
        if not self.has_section(section):
            self.add_section(section)
        self.set(section, option, value)


config = CanariConfigParser()

dconf = path.join(conf)
lconf = path.join(getcwd(), 'canari.conf')

config.read([dconf, lconf])
config.read(config['default/configs'])


# !/usr/bin/env python

import os
from canari.pkgutils.transform import TransformDistribution

from common import canari_main
from framework import SubCommand, Argument


__author__ = 'Nadeem Douba'
__copyright__ = 'Copyright 2012, Canari Project'
__credits__ = []

__license__ = 'GPL'
__version__ = '0.4'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'


def parse_args(args):
    if args.settings_dir:
        args.settings_dir = os.path.realpath(args.settings_dir)
    return args


@SubCommand(
    canari_main,
    help="Uninstalls and unconfigures canari transform packages in Maltego's UI.",
    description="Uninstalls and unconfigures canari transform packages in Maltego's UI."
)
@Argument(
    'package',
    metavar='<package>',
    help='the name of the canari transforms package to uninstall.'
)
@Argument(
    '-s',
    '--settings-dir',
    metavar='[settings dir]',
    default=None,
    help='the path to the Maltego settings directory (automatically detected if excluded)'
)
@Argument(
    '-w',
    '--working-dir',
    metavar='[working dir]',
    default=None,
    help="the path that will be used as the working directory for "
         "the transforms being uninstalled (default: ~/.canari/)"
)
def uninstall_package(args):
    opts = parse_args(args)

    try:
        transform_package = TransformDistribution(opts.package)
        transform_package.uninstall(opts.working_dir, opts.settings_dir)
    except ValueError, e:
        print str(e)
        exit(-1)



#!/usr/bin/env python

import os
from canari.pkgutils.transform import TransformDistribution

from common import canari_main
from framework import SubCommand, Argument


__author__ = 'Nadeem Douba'
__copyright__ = 'Copyright 2012, Canari Project'
__credits__ = []

__license__ = 'GPL'
__version__ = '0.6'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'


@SubCommand(
    canari_main,
    help="Unloads a canari package from Plume.",
    description="Unloads a canari package from Plume."
)
@Argument(
    'package',
    metavar='<package>',
    help='the name of the canari transforms package to unload from Plume.'
)
@Argument(
    '-d',
    '--plume-dir',
    metavar='[www dir]',
    default=os.getcwd(),
    help='the path where Plume is installed.'
)
def unload_plume_package(opts):
    if not os.path.exists(os.path.join(opts.plume_dir, 'plume.py')):
        print('Plume does not appear to be installed in %s.' % opts.plume_dir)
        exit(-1)

    try:
        TransformDistribution(opts.package).configure(opts.plume_dir, load=False, remote=True)
    except ImportError:
        pass

    exit(0)


#!/usr/bin/env python

__author__ = 'Nadeem Douba'
__copyright__ = 'Copyright 2012, Canari Project'
__credits__ = []

__license__ = 'GPL'
__version__ = '0.1'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'


#!/usr/bin/env python

from common import canari_main, project_tree
from framework import SubCommand, Argument

from os import path, rename
from re import sub


__author__ = 'Nadeem Douba'
__copyright__ = 'Copyright 2012, Canari Project'
__credits__ = []

__license__ = 'GPL'
__version__ = '0.3'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'


def parse_args(args):
    if args.transform_dir is None:
        args.transform_dir = project_tree()['transforms']
    return args


@SubCommand(
    canari_main,
    help='Renames a transform in the specified directory and auto-updates __init__.py.',
    description='Renames a transform in the specified directory and auto-updates __init__.py.'
)
@Argument(
    'transform',
    metavar='<old transform name>',
    help='The name of the transform you wish to rename.'
)
@Argument(
    'new_transform',
    metavar='<new transform name>',
    help='The desired name of the transform.'
)
@Argument(
    '-d',
    '--transform-dir',
    metavar='<dir>',
    help='The directory from which you wish to rename the transform.',
    default=None
)
def rename_transform(args):

    opts = parse_args(args)

    initf = path.join(opts.transform_dir, '__init__.py')
    transform = opts.transform
    transformf = path.join(opts.transform_dir, transform if transform.endswith('.py') else '%s.py' % transform )
    dtransform = opts.new_transform
    dtransformf = path.join(opts.transform_dir, dtransform if dtransform.endswith('.py') else '%s.py' % dtransform )

    if not path.exists(initf):
        print ('Directory %s does not appear to be a python package directory... quitting!' % repr(opts.transform_dir))
        exit(-1)
    if not path.exists(transformf):
        print ("Transform %s doesn't exists... quitting" % repr(transformf))
        exit(-1)
    if path.exists(dtransformf):
        print ("Cannot overwrite existing transform %s... quitting" % repr(dtransformf))
        exit(-1)
    if dtransform == transform:
        print ("Nothing to do here... the new name is the same as the old one?")
        exit(-1)

    print ('renaming transform %s to %s...' % (repr(transformf), repr(dtransformf)))
    rename(transformf, dtransformf)

    print ('updating %s' % initf)
    init = file(initf).read()

    with file(initf, mode='wb') as w:
        w.write(
            sub(
                repr(transform),
                repr(dtransform),
                init
            )
        )

    print ('done!')



#!/usr/bin/env python
from collections import OrderedDict

from framework import Argument, SubCommand
from common import canari_main

from csv import reader, DictWriter


__author__ = 'Nadeem Douba'
__copyright__ = 'Copyright 2012, Canari Project'
__credits__ = []

__license__ = 'GPL'
__version__ = '0.4'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'


@SubCommand(
    canari_main,
    help='Convert mixed entity type CSVs to separated CSV sheets.',
    description='Convert mixed entity type CSVs to separated CSV sheets.'
)
@Argument(
    'graph',
    metavar='<graph csv>',
    help='The CSV file containing the output from the mtgx2csv command.'
)
@Argument(
    'prefix',
    metavar='[sheet prefix]',
    nargs='?',
    help='The prefix to prepend to the generated CSV files.'
)
def csv2sheets(opts):

    if not opts.graph.endswith('.csv'):
        print "Invalid file type: %s. Please make sure you run this command against a CSV " \
              "file generated from 'canari mtgx2csv'." % opts.graph
        exit(-1)
    opts.prefix = opts.prefix or opts.graph.split('.', 1)[0]

    sheets = {}
    sheet_headers = {}

    try:
        with file(opts.graph) as csvfile:
            for row in reader(csvfile):
                fv = OrderedDict(column.split('=', 1) for column in row)
                entity_type = fv.pop('Entity Type')
                headers = fv.keys()
                if entity_type not in sheets:
                    sheets[entity_type] = [fv]
                    sheet_headers[entity_type] = headers
                    continue
                else:
                    sheets[entity_type].append(fv)
                if len(headers) > len(sheet_headers[entity_type]):
                    sheet_headers[entity_type].extend([h for h in headers if h not in sheet_headers[entity_type]])

        for entity_type in sheets:
            filename = '%s_%s.csv' % (opts.prefix, entity_type)
            print 'Writing %s sheet to %s...' % (entity_type, filename)
            with open(filename, 'wb') as csvfile:
                csv = DictWriter(csvfile, sheet_headers[entity_type])
                csv.writeheader()
                csv.writerows(sheets[entity_type])
    except IOError, e:
        print 'csv2sheets: %s' % e
        exit(-1)


#!/usr/bin/env python

__author__ = 'Nadeem Douba'
__copyright__ = 'Copyright 2012, Canari Project'
__credits__ = []

__license__ = 'GPL'
__version__ = '0.3'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'

__all__ = [
    'banner',
    'common',
    'create_package',
    'create_profile',
    'create_transform',
    'csv2sheets',
    'debug_transform',
    'delete_transform',
    'framework',
    'generate_entities',
    'help',
    'install_package',
    'install_plume',
    'list_commands',
    'load_plume_package',
    'mtgx2csv',
    'rename_transform',
    'run_server',
    'run_transform',
    'shell',
    'uninstall_package',
    'unload_plume_package',
    'version'
]


#!/usr/bin/env python

from canari.resource import external_resource
from canari.utils.stack import calling_package

from subprocess import PIPE, Popen
import os
import re


__author__ = 'Nadeem Douba'
__copyright__ = 'Copyright 2012, Canari Project'
__credits__ = []

__license__ = 'GPL'
__version__ = '0.3'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'

__all__ = [
    'privileged',
    'specification'
]


def superuser(f):
    f.privileged = True
    return f


def deprecated(f):
    f.deprecated = True
    return f


class configure(object):
    def __init__(self, **kwargs):
        diff = set(['label', 'uuids', 'inputs']).difference(kwargs)
        if diff:
            raise TypeError('Missing transform specification properties: %s' % ', '.join(diff))
        if not isinstance(kwargs['uuids'], list):
            raise TypeError('Expected type list (got %s instead)' % type(kwargs['uuids']).__name__)
        if not isinstance(kwargs['inputs'], list):
            raise TypeError('Expected type list (got %s instead)' % type(kwargs['inputs']))
        kwargs['description'] = kwargs.get('description', '')
        kwargs['debug'] = kwargs.get('debug', False)
        self.function = kwargs.get('cmd', None)
        self.specification = kwargs

    def __call__(self, f):
        if callable(self.function):
            self.function.__dict__.update(f.__dict__)
            f = self.function
        f.__dict__.update(self.specification)
        return f


class ExternalCommand(object):
    def __init__(self, transform_name, transform_args=None, interpreter=None, is_resource=True):
        if transform_args is None:
            transform_args = []
        self._extra_external_args = []

        if interpreter is not None:
            self._extra_external_args.append(interpreter)
            libpath = external_resource(
                os.path.dirname(transform_name),
                '%s.resources.external' % calling_package()
            )
            if interpreter.startswith('perl') or interpreter.startswith('ruby'):
                self._extra_external_args.extend(['-I', libpath])
            elif interpreter.startswith('java'):
                self._extra_external_args.extend(['-cp', libpath])

        if ' ' in transform_name:
            raise ValueError('Transform name %s cannot have spaces.' % repr(transform_name))
        elif not is_resource:
            self._extra_external_args.append(transform_name)
        else:
            self._extra_external_args.append(
                external_resource(
                    transform_name,
                    '%s.resources.external' % calling_package()
                )
            )

        if isinstance(transform_args, basestring):
            self._extra_external_args = re.split(r'\s+', transform_args)
        else:
            self._extra_external_args.extend(transform_args)

    def __call__(self, request, request_xml):
        args = [request.value]
        if isinstance(request.params, list) and request.params:
            args.extend(request.params)
        if request.fields:
            args.append('#'.join(['%s=%s' % (k, v) for k, v in request.fields.iteritems()]))
        if isinstance(request_xml, basestring):
            p = Popen(self._extra_external_args + list(args), stdin=PIPE, stdout=PIPE)
            out, err = p.communicate(request_xml)
            return out
        p = Popen(self._extra_external_args + list(args))
        p.communicate()
        exit(p.returncode)



#!/usr/bin/env python

from distutils.command.install import install
from distutils.dist import Distribution
from argparse import Action
from datetime import datetime
from string import Template
import unicodedata
import subprocess
import sys
import os
import re

from pkg_resources import resource_filename

from canari.commands.framework import Command
from canari.config import CanariConfigParser


__author__ = 'Nadeem Douba'
__copyright__ = 'Copyright 2012, Canari Project'
__credits__ = ['Andrew Udvare']

__license__ = 'GPL'
__version__ = '0.6'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'


class ParseFieldsAction(Action):
    """
    Custom argparse action to parse arguments for the run- and debug-transform commands. This ensures that all
    positional arguments are parsed and stored correctly.
    """

    def __call__(self, parser, namespace, values, option_string=None):
        # Does the value argument have an equals ('=') sign that is not escaped and is the params argument populated?
        if namespace.params and re.search(r'(?<=[^\\])=', namespace.value):
            # if so, apply fix and pop last element of namespace.params into namespace.value
            # and copy what was in namespace.value into namespace.fields to fix everything
            values = namespace.value
            namespace.value = namespace.params.pop().replace('\=', '=')
            namespace.fields = values
            # Next parse our fields argument into a dictionary
            fs = re.split(r'(?<=[^\\])#', values)
            if fs:
                namespace.fields = dict(
                    map(
                        lambda x: [
                            c.replace('\#', '#').replace('\=', '=').replace('\\\\', '\\')
                            for c in re.split(r'(?<=[^\\])=', x, 1)
                        ],
                        fs
                    )
                )


@Command(description='Centralized Canari Management System')
def canari_main(opts):
    """
    This is the main function for the Canari commander. Nothing special here.
    """
    fix_pypath()
    opts.command_function(opts)


def get_bin_dir():
    """
    Returns the absolute path of the installation directory for the Canari scripts.
    """
    d = install(Distribution())
    d.finalize_options()
    return d.install_scripts


def to_utf8(s):
    return unicodedata.normalize('NFKD', unicode(s)).encode('ascii', 'ignore')


def sudo(args):
    p = subprocess.Popen([os.path.join(get_bin_dir(), 'pysudo')] + args, stdin=subprocess.PIPE)
    p.communicate()
    return p.returncode


def uproot():
    if os.name == 'posix' and not os.geteuid():
        login = os.getlogin()

        if login != 'root':
            print 'Why are you using root to run this command? You should be using %s! Bringing you down...' % login
            import pwd

            user = pwd.getpwnam(login)
            os.setgid(user.pw_gid)
            os.setuid(user.pw_uid)


def read_template(name, values):
    t = Template(file(resource_filename('canari.resources.template', '%s.plate' % name)).read())
    return t.substitute(**values)


def write_template(dst, data):
    print('creating file %s...' % dst)
    with file(dst, mode='wb') as w:
        w.write(data)


def generate_all(*args):
    return "\n\n__all__ = [\n    '%s'\n]" % "',\n    '".join(args)


def build_skeleton(*args):
    for d in args:
        if isinstance(d, list):
            d = os.sep.join(d)
        print('creating directory %s' % d)
        os.mkdir(d)


def fix_pypath():
    if '' not in sys.path:
        sys.path.insert(0, '')


def fix_binpath(paths):
    if paths is not None and paths:
        if isinstance(paths, basestring):
            os.environ['PATH'] = paths
        elif isinstance(paths, list):
            os.environ['PATH'] = os.pathsep.join(paths)


def import_transform(script):
    return __import__(script, globals(), locals(), ['dotransform'])


def import_package(package):
    return __import__(package, globals(), locals(), ['*'])


def init_pkg():
    root = project_root()

    if root is not None:
        conf = os.path.join(root, '.canari')
        if os.path.exists(conf):
            c = CanariConfigParser()
            c.read(conf)
            return {
                'author': c['metadata/author'],
                'email': c['metadata/email'],
                'maintainer': c['metadata/maintainer'],
                'project': c['metadata/project'],
                'year': datetime.now().year
            }

    return {
        'author': '',
        'email': '',
        'maintainer': '',
        'project': '',
        'year': datetime.now().year
    }


def project_root():
    marker = '.canari'
    for i in range(0, 5):
        if os.path.exists(marker) and os.path.isfile(marker):
            return os.path.dirname(os.path.realpath(marker))
        marker = '..%s%s' % (os.sep, marker)
    raise ValueError('Unable to determine project root.')


def project_tree():
    root = project_root()

    tree = dict(
        root=root,
        src=None,
        pkg=None,
        resources=None,
        transforms=None
    )

    for base, dirs, files in os.walk(root):
        if base.endswith('src'):
            tree['src'] = base
        elif 'resources' in dirs:
            tree['pkg'] = base
        elif base.endswith('resources'):
            tree['resources'] = base
        elif base.endswith('transforms'):
            tree['transforms'] = base

    return tree


def parse_bool(question, default=True):
    choices = 'Y/n' if default else 'y/N'
    default = 'Y' if default else 'N'
    while True:
        ans = raw_input('%s [%s]: ' % (question, choices)).upper() or default
        if ans.startswith('Y'):
            return True
        elif ans.startswith('N'):
            return False
        else:
            print('Invalid selection (%s) must be either [y]es or [n]o.' % ans)


def parse_int(question, choices, default=0):
    while True:
        for i, c in enumerate(choices):
            print('[%d] - %s' % (i, c))
        ans = raw_input('%s [%d]: ' % (question, default)) or default
        try:
            ans = int(ans)
            if not 0 <= ans <= i:
                raise ValueError
            return ans
        except ValueError:
            print('Invalid selection (%s) must be an integer between 0 and %d.' % (ans, i))


def parse_str(question, default):
    return raw_input('%s [%s]: ' % (question, default)) or default


class pushd(object):
    """
    Ripped from here: https://gist.github.com/Tatsh/7131812
    """

    def __init__(self, dir_name):
        self.cwd = os.path.realpath(dir_name)
        self.original_dir = None

    def __enter__(self):
        self.original_dir = os.getcwd()
        os.chdir(self.cwd)
        return self

    def __exit__(self, type_, value, tb):
        os.chdir(self.original_dir)


#!/usr/bin/env python

from common import canari_main, project_tree
from framework import SubCommand, Argument

from os import path, unlink
from re import sub


__author__ = 'Nadeem Douba'
__copyright__ = 'Copyright 2012, Canari Project'
__credits__ = []

__license__ = 'GPL'
__version__ = '0.3'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'


def parse_args(args):
    if args.transform_dir is None:
        args.transform_dir = project_tree()['transforms']
    if args.transform in ['common', 'common.py']:
        print "Error: 'common' is not a transform module. Cannot delete this module."
        exit(-1)
    return args


@SubCommand(
    canari_main,
    help='Deletes a transform in the specified directory and auto-updates __init__.py.',
    description='Deletes a transform in the specified directory and auto-updates __init__.py.'
)
@Argument(
    'transform',
    metavar='<transform name>',
    help='The name of the transform you wish to delete.'
)
@Argument(
    '-d',
    '--transform-dir',
    metavar='<dir>',
    help='The directory from which you wish to delete the transform.',
    default=None
)
def delete_transform(args):

    opts = parse_args(args)

    initf = path.join(opts.transform_dir, '__init__.py')
    transform = opts.transform
    transformf = path.join(opts.transform_dir, transform if transform.endswith('.py') else '%s.py' % transform )

    if not path.exists(initf):
        print ('Directory %s does not appear to be a python package directory... quitting!' % repr(opts.transform_dir))
        exit(-1)
    if not path.exists(transformf):
        print ("Transform %s doesn't exists... quitting" % repr(transformf))
        exit(-1)

    print ("deleting transform %s..." % repr(transformf))
    unlink(transformf)

    print ('updating %s' % initf)
    init = file(initf).read()

    with file(initf, mode='wb') as w:
        w.write(
            sub(
                r'\s*%s,?' % repr(transform),
                '',
                init
            )
        )

    print ('done!')



#!/usr/bin/env python

from message import Entity, EntityField, EntityFieldType

__author__ = 'Nadeem Douba'
__copyright__ = 'Copyright 2012, Canari Project'
__credits__ = []

__license__ = 'GPL'
__version__ = '0.2'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'

__all__ = [
    'Affiliation',
    'Bebo',
    'Facebook',
    'Flickr',
    'Linkedin',
    'MySpace',
    'Orkut',
    'Spock',
    'Twitter',
    'WikiEdit',
    'Zoominfo',
    'Alias',
    'AS',
    'Banner',
    'BuiltWithTechnology',
    'Device',
    'DNSName',
    'Document',
    'Domain',
    'EmailAddress',
    'FacebookObject',
    'File',
    'GPS',
    'Image',
    'IPv4Address',
    'Location',
    'MXRecord',
    'Netblock',
    'NominatimLocation',
    'NSRecord',
    'Person',
    'PhoneNumber',
    'Phrase',
    'Port',
    'Service',
    'Twit',
    'URL',
    'Vulnerability',
    'Webdir',
    'Website',
    'WebTitle'
]


class Unknown(Entity):
    pass


@EntityField(name='properties.gps', propname='gps', displayname='GPS Co-ordinate', is_value=True)
@EntityField(name='latitude', displayname='Latitude', type=EntityFieldType.Float)
@EntityField(name='longitude', displayname='Longitude', type=EntityFieldType.Float)
class GPS(Entity):
    pass


@EntityField(name='properties.device', displayname='Device')
class Device(Entity):
    pass


@EntityField(name='properties.builtwithtechnology', propname='builtwith', displayname='BuiltWith Technology')
class BuiltWithTechnology(Entity):
    pass



@EntityField(name='fqdn', displayname='Domain Name', is_value=True)
@EntityField(name='whois-info', propname='whois_info', displayname='WHOIS Info')
class Domain(Entity):
    _v2type_ = 'Domain'
    _fields_to_properties_ = {
        'whois': 'whois_info'
    }


@EntityField(name='fqdn', displayname='DNS Name', is_value=True)
class DNSName(Entity):
    _v2type_ = 'DNSName'


@EntityField(name='mxrecord.priority', propname='priority', type=EntityFieldType.Integer)
class MXRecord(DNSName):
    _v2type_ = 'MXRecord'


class NSRecord(DNSName):
    _v2type_ = 'NSRecord'


@EntityField(name='ipv4-address', propname='ipv4address', displayname='IP Address', is_value=True)
@EntityField(name='ipaddress.internal', propname='internal', displayname='Internal', type=EntityFieldType.Bool)
class IPv4Address(Entity):
    _v2type_ = 'IPAddress'


@EntityField(name='ipv4-range', propname='ipv4range', displayname='IP Range', is_value=True)
class Netblock(Entity):
    _v2type_ = 'Netblock'


@EntityField(name='as.number', propname='number', displayname='AS Number', type=EntityFieldType.Integer, is_value=True)
class AS(Entity):
    _v2type_ = 'ASNumber'


@EntityField(name='website.ssl-enabled', propname='ssl_enabled', displayname='SSL Enabled', type=EntityFieldType.Bool)
@EntityField(name='fqdn', displayname='Website', is_value=True)
@EntityField(name='ports', displayname='Ports', type=EntityFieldType.Integer)
class Website(Entity):
    _v2type_ = 'Website'


@EntityField(name='short-title', propname='short_title', displayname='Short title', is_value=True)
@EntityField(name='url', displayname='URL')
@EntityField(name='title', displayname='Title')
class URL(Entity):
    _v2type_ = 'URL'
    _fields_to_properties_ = {
        'maltego.v2.value.property': 'shorttitle',
        'theurl': 'url',
        'fulltitle': 'title'
    }


@EntityField(name='text', displayname='Text', is_value=True)
class Phrase(Entity):
    _v2type_ = 'Phrase'


@EntityField(name='title', displayname='Title')
@EntityField(name='url', displayname='URL', is_value=True)
@EntityField(name='document.metadata', propname='metadata', displayname='Meta-Data')
class Document(Entity):
    _v2type_ = 'Document'
    _fields_to_properties_ = {
        'maltego.v2.value.property': 'title',
        'link': 'url',
        'metainfo': 'metadata'
    }


@EntityField(name='person.lastname', propname='lastname', displayname='Surname')
@EntityField(name='person.firstnames', propname='firstnames', displayname='First Names')
@EntityField(name='person.fullname', propname='fullname', displayname='Full Name', is_value=True)
class Person(Entity):
    _v2type_ = 'Person'
    _fields_to_properties_ = {
        'lastname': 'lastname',
        'firstname': 'firstnames'
    }


@EntityField(name='email', displayname='Email Address', is_value=True)
class EmailAddress(Entity):
    _v2type_ = 'EmailAddress'


@EntityField(name='content', displayname='Content')
@EntityField(name='pubdate', displayname='Date published')
@EntityField(name='img_link', displayname='Image Link')
@EntityField(name='author', displayname='Author')
@EntityField(name='title', displayname='Title')
@EntityField(name='author_uri', displayname='Author URI')
@EntityField(name='twit.name', propname='name', displayname='Twit', is_value=True)
@EntityField(name='id', displayname='Twit ID')
class Twit(Entity):
    _v2type_ = 'Twit'
    _fields_to_properties_ = {
        'imglink': 'img_link'
    }


@EntityField(name='person.name', propname='person_name', displayname='Name', is_value=True)
@EntityField(name='affiliation.uid', propname='uid', displayname='UID')
@EntityField(name='affiliation.network', propname='network', displayname='Network')
@EntityField(name='affiliation.profile-url', propname='profile_url', displayname='Profile URL')
class Affiliation(Entity):
    _namespace_ = 'maltego.affiliation'
    _fields_to_properties_ = {
        'network': 'network',
        'uid': 'uid',
        'profile_url': 'profile_url'
    }


class Bebo(Affiliation):
    _v2type_ = 'AffiliationBebo'


class Facebook(Affiliation):
    _v2type_ = 'AffiliationFacebook'


class Flickr(Affiliation):
    _v2type_ = 'AffiliationFlickr'


class Linkedin(Affiliation):
    _v2type_ = 'AffiliationLinkedin'


class MySpace(Affiliation):
    _v2type_ = 'AffiliationMySpace'


class Orkut(Affiliation):
    _v2type_ = 'AffiliationOrkut'


@EntityField(name='twitter.number', propname='number', displayname='Twitter Number', type=EntityFieldType.Integer)
@EntityField(name='twitter.screen-name', propname='screenname', displayname='Screen Name')
@EntityField(name='twitter.friendcount', propname='friendcount', displayname='Friend Count',
             type=EntityFieldType.Integer)
@EntityField(name='person.fullname', propname='fullname', displayname='Real Name')
class Twitter(Affiliation):
    _v2type_ = 'AffiliationTwitter'


class Zoominfo(Affiliation):
    pass


class WikiEdit(Affiliation):
    pass


@EntityField(name='spock.websites', propname='websites', displayname='Listed Websites')
class Spock(Affiliation):
    _v2type_ = 'AffiliationSpock'


@EntityField(name='properties.facebookobject', propname='object', displayname='Facebook Object')
class FacebookObject(Entity):
    pass


@EntityField(name='city', displayname='City')
@EntityField(name='countrycode', displayname='Country Code')
@EntityField(name='location.area', propname='area', displayname='Area')
@EntityField(name='country', displayname='Country')
@EntityField(name='longitude', displayname='Longitude', type=EntityFieldType.Float)
@EntityField(name='latitude', displayname='Latitude', type=EntityFieldType.Float)
@EntityField(name='streetaddress', displayname='Street Address')
@EntityField(name='location.areacode', propname='areacode', displayname='Area Code')
@EntityField(name='location.name', propname='name', displayname='Name', is_value=True)
class Location(Entity):
    _v2type_ = 'Location'
    _fields_to_properties_ = {
        'countrysc': 'countrycode',
        'area': 'area',
        'long': 'longitude',
        'lat': 'latitude'
    }


@EntityField(name='properties.nominatimlocation', propname='nominatim', displayname='Nominatim Location', is_value=True)
class NominatimLocation(Entity):
    pass


@EntityField(name='phonenumber.areacode', propname='areacode', displayname='Area Code')
@EntityField(name='phonenumber.lastnumbers', propname='lastnumbers', displayname='Last Digits')
@EntityField(name='phonenumber.citycode', propname='citycode', displayname='City Code')
@EntityField(name='phonenumber', displayname='Phone Number', is_value=True)
@EntityField(name='phonenumber.countrycode', propname='countrycode', displayname='Country Code')
class PhoneNumber(Entity):
    _v2type_ = 'PhoneNumber'
    _fields_to_properties_ = {
        'countrycode': 'countrycode',
        'citycode': 'citycode',
        'areacode': 'areacode',
        'lastnumbers': 'lastnumbers'
    }


@EntityField(name='properties.alias', propname='alias', displayname='Alias')
class Alias(Entity):
    pass


@EntityField(name='properties.image', propname='description', displayname='Description')
@EntityField(name='fullImage', propname='url', displayname='URL')
class Image(Entity):
    pass


@EntityField(name='source', displayname='Source')
@EntityField(name='description', displayname='Description')
class File(Entity):
    pass


@EntityField(name='banner.text', propname='text', displayname='Banner', is_value=True)
class Banner(Entity):
    _v2type_ = 'Banner'


@EntityField(name='port.number', propname='number', displayname='Ports', is_value=True)
class Port(Entity):
    _v2type_ = 'Port'


@EntityField(name='service.name', propname='name', displayname='Description', is_value=True)
@EntityField(name='banner.text', propname='banner', displayname='Service Banner')
@EntityField(name='port.number', propname='port', displayname='Ports')
class Service(Entity):
    _v2type_ = 'Service'


@EntityField(name='vulnerability.id', propname='id', displayname='ID', is_value=True)
class Vulnerability(Entity):
    _v2type_ = 'Vuln'


@EntityField(name='directory.name', propname='name', displayname='Name', is_value=True)
class Webdir(Entity):
    _v2type_ = 'Webdir'


@EntityField(name='title', displayname='Title', is_value=True)
class WebTitle(Entity):
    _v2type_ = 'WebTitle'


#!/usr/bin/env python

from imghdr import what
from utils.stack import calling_package
from pkg_resources import resource_filename, resource_listdir, resource_isdir, resource_exists

__author__ = 'Nadeem Douba'
__copyright__ = 'Copyright 2012, Canari Project'
__credits__ = []

__license__ = 'GPL'
__version__ = '0.5'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'

__all__ = [
    'external_resource',
    'image_resource',
    'icon_resource',
    'image_resources',
    'resource',
    'conf'
]


etc = 'canari.resources.etc'


def imageicon(pkg, name):
    name = resource_filename(pkg, name)
    if name[0] != '/':
        return 'file:///%s' % name
    return 'file://%s' % name


def imagepath(pkg, name):
    return resource_filename(pkg, name)


def external_resource(name, pkg=None):
    if pkg is None:
        pkg = '%s.resources.external' % calling_package()
    return resource_filename(pkg, name)


def image_resource(name, pkg=None):
    if pkg is None:
        pkg = '%s.resources.images' % calling_package()
    return imagepath(pkg, name)


def icon_resource(name, pkg=None):
    if pkg is None:
        pkg = '%s.resources.images' % calling_package()
    return imageicon(pkg, name)


def image_resources(pkg=None, directory='resources'):
    if pkg is None:
        pkg = calling_package()
    pkg_dir = '%s.%s' % (pkg, directory)
    images = []
    for i in resource_listdir(pkg, directory):
        fname = resource_filename(pkg_dir, i)
        if resource_isdir(pkg_dir, i):
            images.extend(image_resources(pkg_dir, i))
        elif what(fname) is not None:
            images.append(fname)
    return images


def resource(name, pkg=None):
    if pkg is None:
        pkg = '%s.resources' % calling_package()
    resource = resource_filename(pkg, name)
    if not resource_exists(resource):
        raise OSError('Resource does not exist %s' % repr(resource))
    return resource

# etc
conf = resource_filename(etc, 'canari.conf')



#!/usr/bin/env python

import time
from canari.xmltools.oxml import MaltegoElement, fields as fields_


__author__ = 'Nadeem Douba'
__copyright__ = 'Copyright 2012, Canari Project'
__credits__ = []

__license__ = 'GPL'
__version__ = '0.3'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'

__all__ = [
    'TransformAdapter',
    'VisibilityType',
    'BuiltInTransformSets',
    'Set',
    'Transform',
    'TransformSet',
    'InputConstraint',
    'OutputEntity',
    'InputEntity',
    'PropertyType',
    'TransformProperty',
    'TransformPropertySetting',
    'CmdLineTransformProperty',
    'CmdLineTransformPropertySetting',
    'CmdParmTransformProperty',
    'CmdParmTransformPropertySetting',
    'CmdCwdTransformProperty',
    'CmdCwdTransformPropertySetting',
    'CmdDbgTransformProperty',
    'CmdDbgTransformPropertySetting',
    'TransformSettings',
    'Protocol',
    'AuthenticationType',
    'Authentication',
    'MaltegoServer',
    'EntityCategory',
    'attr',
    'fileobject',
    'attributes',
    'Properties',
    'MaltegoTransform'
]

class TransformAdapter(object):
    Local = 'com.paterva.maltego.transform.protocol.v2.LocalTransformAdapterV2'
    Remote = 'com.paterva.maltego.transform.protocol.v2.RemoteTransformAdapterV2'


class VisibilityType(object):
    Public = 'public'
    Private = 'private'


class BuiltInTransformSets(object):
    ConvertToDomain = "Convert to Domain"
    DomainsUsingMXNS = "Domains using MX NS"
    FindOnWebpage = "Find on webpage"
    RelatedEmailAddresses = "Related Email addresses"
    DNSFromDomain = "DNS from Domain"
    EmailAddressesFromDomain = "Email addresses from Domain"
    IPOwnerDetail = "IP owner detail"
    ResolveToIP = "Resolve to IP"
    DNSFromIP = "DNS from IP"
    EmailAddressesFromPerson = "Email addresses from Person"
    InfoFromNS = "Info from NS"
    DomainFromDNS = "Domain From DNS"
    FilesAndDocumentsFromDomain = "Files and Documents from Domain"
    LinksInAndOutOfSite = "Links in and out of site"
    DomainOwnerDetail = "Domain owner detail"
    FilesAndDocumentsFromPhrase = "Files and Documents from Phrase"


class Set(MaltegoElement):
    name = fields_.String()


class Transform(MaltegoElement):
    name = fields_.String()


class TransformSet(MaltegoElement):

    name = fields_.String()
    description = fields_.String(default='')
    transforms = fields_.List(Transform, tagname='Transforms')

    def appendelement(self, other):
        if isinstance(other, Transform):
            self.transforms.append(other)

    def removeelement(self, other):
        if isinstance(other, Transform):
            self.transforms.remove(other)


class InputConstraint(MaltegoElement):

    class meta:
        tagname = 'Entity'

    type = fields_.String()
    min = fields_.Integer(default=1)
    max = fields_.Integer(default=1)


class OutputEntity(InputConstraint):

    class meta:
        tagname = 'Entity'


class InputEntity(InputConstraint):

    class meta:
        tagname = 'Entity'


class PropertyType(object):
    String = 'string'
    Boolean = 'boolean'
    Integer = 'int'


class TransformProperty(MaltegoElement):

    class meta:
        tagname = 'Property'

    defaultvalue = fields_.String(tagname='DefaultValue', required=False)
    samplevalue = fields_.String(tagname='SampleValue', default='')
    abstract = fields_.Boolean(default=False)
    description = fields_.String(default='')
    displayname = fields_.String(attrname='displayName')
    hidden = fields_.Boolean(default=False)
    name = fields_.String()
    nullable = fields_.Boolean(default=False)
    readonly = fields_.Boolean(default=False)
    popup = fields_.Boolean(default=False)
    type = fields_.String(default=PropertyType.String)
    visibility = fields_.String(default=VisibilityType.Public)


class TransformPropertySetting(MaltegoElement):

    class meta:
        tagname='Property'

    name = fields_.String()
    popup = fields_.Boolean(default=False)
    type = fields_.String(default=PropertyType.String)
    value = fields_.String(tagname=".")


def CmdLineTransformProperty(cmd=''):
    return TransformProperty(
        name='transform.local.command',
        defaultvalue=cmd,
        displayname='Command line',
        description='The command to execute for this transform'
    )


def CmdLineTransformPropertySetting(cmd=''):
    return TransformPropertySetting(
        name='transform.local.command',
        value=cmd
    )


def CmdParmTransformProperty(params=''):
    return TransformProperty(
        name='transform.local.parameters',
        defaultvalue=params,
        displayname='Command parameters',
        description='The parameters to pass to the transform command'
    )


def CmdParmTransformPropertySetting(params=''):
    return TransformPropertySetting(
        name='transform.local.parameters',
        value=params
    )


def CmdCwdTransformProperty(cwd=''):
    return TransformProperty(
        name='transform.local.working-directory',
        defaultvalue=cwd,
        displayname='Working directory',
        description='The working directory used when invoking the executable',
        samplevalue='/'
    )


def CmdCwdTransformPropertySetting(cwd=''):
    return TransformPropertySetting(
        name='transform.local.working-directory',
        value=cwd
    )


def CmdDbgTransformProperty(dbg=False):
    return TransformProperty(
        name='transform.local.debug',
        defaultvalue=str(dbg).lower(),
        displayname='Show debug info',
        description="When this is set, the transform's text output will be printed to the output window",
        samplevalue='false',
        type=PropertyType.Boolean
    )


def CmdDbgTransformPropertySetting(dbg=False):
    return TransformPropertySetting(
        name='transform.local.debug',
        value=str(dbg).lower(),
        type=PropertyType.Boolean
    )


class TransformSettings(MaltegoElement):

    enabled = fields_.Boolean(default=True)
    accepted = fields_.Boolean(default=False, attrname='disclaimerAccepted')
    showhelp = fields_.Boolean(default=True, attrname='showHelp')
    properties = fields_.List(TransformPropertySetting, tagname='Properties')

    def appendelement(self, other):
        if isinstance(other, TransformPropertySetting):
            self.properties.append(other)

    def removeelement(self, other):
        if isinstance(other, TransformPropertySetting):
            self.properties.remove(other)


class Protocol(MaltegoElement):

    version = fields_.String(default='2.0')


class AuthenticationType(object):
    Anonymous = 'none'
    Mac = 'mac'
    License = 'license'


class Authentication(MaltegoElement):

    type = fields_.String(default=AuthenticationType.Anonymous)


class MaltegoServer(MaltegoElement):

    name = fields_.String(default='Local')
    enabled = fields_.Boolean(default=True)
    description = fields_.String(default='Local transforms hosted on this machine')
    url = fields_.String(default='http://localhost')
    lastsync = fields_.String(tagname='LastSync', default=time.strftime('%Y-%m-%d'))
    protocol = fields_.Model(Protocol)
    authentication = fields_.Model(Authentication)
    transforms = fields_.List(Transform, tagname='Transforms')

    def appendelement(self, other):
        if isinstance(other, Transform):
            self.transforms.append(other)

    def removeelement(self, other):
        if isinstance(other, Transform):
            self.transforms.remove(other)


class EntityCategory(MaltegoElement):

    name = fields_.String()


class attr(MaltegoElement):
    name = fields_.String()
    stringvalue = fields_.String(required=False)
    boolvalue = fields_.Boolean(required=False)


class fileobject(MaltegoElement):

    name = fields_.String()
    attrs = fields_.Dict(attr, key='name')

    def appendelement(self, other):
        if isinstance(other, attr):
            self.attrs[other.name] = other

    def removeelement(self, other):
        if isinstance(other, attr):
            del self.attrs[other.name]


class attributes(MaltegoElement):

    version = fields_.String(default='1.0')
    fileobjects = fields_.Dict(fileobject, key='name')

    def appendelement(self, other):
        if isinstance(other, fileobject):
            self.fileobjects[other.name] = other

    def removeelement(self, other):
        if isinstance(other, fileobject):
            del self.fileobjects[other.name]


class Properties(MaltegoElement):

    fields = fields_.Dict(TransformProperty, key='name', tagname='Fields')

    def appendelement(self, other):
        if isinstance(other, TransformProperty):
            self.fields[other.name] = other

    def removeelement(self, other):
        if isinstance(other, TransformProperty):
            del self.fields[other.name]


class MaltegoTransform(MaltegoElement):

    name = fields_.String()
    displayname = fields_.String(attrname='displayName', default='')
    abstract = fields_.String(default=False)
    template = fields_.Boolean(default=False)
    visibility = fields_.String(default=VisibilityType.Public)
    description = fields_.String(default='')
    helpurl = fields_.String(attrname='helpURL', default='')
    author = fields_.String(default='')
    owner = fields_.String(default='')
    locrel = fields_.String(attrname='locationRelevance', default='global')
    version = fields_.String(default='1.0')
    requireinfo = fields_.Boolean(default=False, attrname='requireDisplayInfo')
    adapter = fields_.String(tagname='TransformAdapter', default=TransformAdapter.Local)
    properties = fields_.Model(Properties)
    input = fields_.List(InputConstraint, tagname='InputConstraints', required=False)
    output = fields_.List(OutputEntity, tagname='OutputEntities', required=False)
    help = fields_.CDATA(tagname='Help', default='')
    disclaimer = fields_.CDATA(tagname='Disclaimer', default='')
    sets = fields_.List(Set, tagname='defaultSets')
    stealthlevel = fields_.Integer(tagname='StealthLevel', default=0)
    authenticator = fields_.String(tagname='Authenticator', required=False)

    def appendelement(self, other):
        if isinstance(other, Set):
            self.sets.append(other)
        elif isinstance(other, TransformProperty):
            self.properties.fields_.append(other)
        elif isinstance(other, InputConstraint) or isinstance(other, InputEntity):
            self.input.append(other)
        elif isinstance(other, OutputEntity):
            self.output.append(other)

    def removeelement(self, other):
        if isinstance(other, Set):
            self.sets.remove(other)
        if isinstance(other, TransformProperty):
            self.properties.fields_.remove(other)
        elif isinstance(other, InputConstraint) or isinstance(other, InputEntity):
            self.input.remove(other)
        elif isinstance(other, OutputEntity):
            self.output.remove(other)


class Field(MaltegoElement):

    name = fields_.String()
    type = fields_.String()
    nullable = fields_.Boolean(default=True)
    hidden = fields_.Boolean(default=False)
    readonly = fields_.Boolean(default=False)
    description = fields_.String(required=False)
    displayname = fields_.String(attrname='displayName')
    defaultvalue = fields_.String(tagname='DefaultValue', required=False)
    samplevalue = fields_.String(tagname='SampleValue', default='')


class Groups(MaltegoElement):
    pass


class EntityProperties(MaltegoElement):

    class meta:
        tagname = 'Properties'

    value = fields_.String(required=False)
    groups = fields_.Model(Groups, required=False)
    fields = fields_.Dict(Field, key='name', tagname='Fields', required=False)


class RegexGroup(MaltegoElement):

    property = fields_.String(required=False)


class Converter(MaltegoElement):

    value = fields_.CDATA(default='', tagname='Value', required=False)
    regexgroups = fields_.Dict(RegexGroup, key='property', required=False)


class MaltegoEntity(MaltegoElement):

    id = fields_.String()
    displayname = fields_.String(attrname='displayName', required=False)
    plural = fields_.String(attrname='displayNamePlural', required=False)
    description = fields_.String(default='', required=False)
    category = fields_.String(required=False)
    smallicon = fields_.String(attrname='smallIconResource', required=False)
    largeicon = fields_.String(attrname='largeIconResource', required=False)
    allowedRoot = fields_.Boolean(default=True, required=False)
    conversion_order = fields_.Integer(attrname='conversionOrder', default=2147483647, required=False)
    visible = fields_.Boolean(default=True, required=False)
    largeicontag = fields_.String(tagname='Icon', required=False)
    smallicontag = fields_.String(tagname='SmallIcon', required=False)
    converter = fields_.Model(Converter, required=False)
    properties = fields_.Model(EntityProperties, required=False)



#!/usr/bin/env python

# Builtin imports
from ConfigParser import NoSectionError
import traceback
from urlparse import urljoin
import hashlib
import sys
import os
import re

# Third-party imports
from canari.pkgutils.transform import TransformDistribution
from flask import Flask, Response, request

# Canari imports
import canari.config as _config
from canari.maltego.utils import get_transform_version
from canari.maltego.message import (MaltegoMessage, MaltegoTransformResponseMessage, MaltegoTransformExceptionMessage,
                                    MaltegoException)

# Monkey patch our resource lib to automatically rewrite icon urls
import canari.resource
imgres = canari.resource.image_resource
canari.resource.image_resource = \
    lambda name, pkg=None: '%sstatic/%s' % (request.host_url, hashlib.md5(imgres(name, pkg)).hexdigest())
canari.resource.icon_resource = canari.resource.image_resource
callpkg = canari.resource.calling_package
canari.resource.calling_package = lambda frame=4: callpkg(frame)

__author__ = 'Nadeem Douba'
__copyright__ = 'Copyright 2012, Canari Project'
__credits__ = []

__license__ = 'GPL'
__version__ = '0.4'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'


class Canari(Flask):

    four_o_four = 'Whaaaaat?'

    def __init__(self, import_name):
        super(Canari, self).__init__(import_name)
        self.transforms = {}
        self.resources = []
        self._initialize()

    def _copy_images(self, pkg):
        if pkg.endswith('.transforms'):
            pkg = pkg.replace('.transforms', '')
        for i in canari.resource.image_resources(pkg):
            dname = 'static/%s' % hashlib.md5(i).hexdigest()
            self.resources.append(dname)
            if not os.path.exists(dname):
                print 'Copying %s to %s...' % (i, dname)
                with open(i, mode='rb') as src:
                    with open(dname, mode="wb") as dst:
                        dst.write(src.read())

    def _initialize(self):
        # Flask application container reload hack.
        reload(_config)

        packages = None

        # Read packages that are to be loaded at runtime
        try:
            packages = _config.config['remote/packages']
        except NoSectionError:
            sys.stderr.write('Exiting... You did not specify a [remote] section and a "packages" '
                             'option in your canari.conf file!\n')
            exit(-1)

        # Is packages not blank
        if not packages:
            sys.stderr.write('Exiting... You did not specify any transform packages to load in your canari.conf file!\n')
            exit(-1)
        elif isinstance(packages, basestring):
            packages = [packages]

        # Create the static directory for static file loading
        if not os.path.exists('static'):
            os.mkdir('static', 0755)

        # Iterate through the list of packages to load
        for p in packages:
            # Copy all the image resource files in case they are used as entity icons
            self._copy_images(p)

            distribution = TransformDistribution(p)

            sys.stderr.write('Loading transform package %s\n' % repr(p))

            for transform in distribution.remote_transforms:
                transform_name = transform.__name__
                sys.stderr.write('Loading transform %s at /%s...\n' % (repr(transform_name), transform_name))
                # Should the transform be publicly available?
                # Does it conform to V2 of the Canari transform signature standard?
                if get_transform_version(transform.dotransform) == 2:
                    sys.stderr.write('ERROR: Plume does not support V2 Canari transforms (%s). Please update to V3.'
                                     ' See http://www.canariproject.com/4-3-transform-development-quick-start/ for'
                                     ' more details.\n' % repr(transform_name))
                    exit(-1)
                    # Does the transform need to be executed as root? If so, is this running in mod_wsgi? Yes = Bad!
                elif os.name == 'posix' and hasattr(transform.dotransform, 'privileged') and \
                        os.geteuid() and __name__.startswith('_mod_wsgi_'):
                    sys.stderr.write('WARNING: mod_wsgi does not allow applications to run with root privileges. '
                                     'Transform %s ignored...\n' % repr(transform_name))
                    continue

                # So everything is good, let's register our transform with the global transform registry.
                inputs = {}
                if hasattr(transform.dotransform, 'inputs'):
                    for category, entity_type in transform.dotransform.inputs:
                        inputs[entity_type._type_] = entity_type
                        inputs[entity_type._v2type_] = entity_type
                self.transforms[transform_name] = (transform.dotransform, inputs)


# Create our Flask app.
app = Canari(__name__)


def croak(error_msg):
    """Throw an exception in the Maltego GUI containing error_msg."""
    return MaltegoMessage(
        message=MaltegoTransformExceptionMessage(
            exceptions=[
                MaltegoException(error_msg)
            ]
        )
    ).render()


def message(m):
    """Write a MaltegoMessage to stdout and exit successfully"""
    v = None
    if isinstance(m, str):
        # Let's make sure that we're not spewing out local file system information ;)
        for url in re.findall("<iconurl>\s*(file://[^\s<]+)\s*</iconurl>(?im)", m):
            path = 'static/%s' % hashlib.md5(url[7:]).hexdigest()
            new_url = urljoin(request.host_url, path)
            m.replace(url, new_url, 1)
        v = m
    else:
        v = MaltegoMessage(message=m).render()
    return Response(v, status=200, mimetype='text/html')


def dotransform(transform, valid_input_entity_types):
    try:
        # Get the body of the request
        request_str = request.data

        # Let's get an XML object tree
        maltego_request = MaltegoMessage.parse(request_str).message

        # Get the entity being passed in.
        e = maltego_request.entity
        entity_type = e.type

        if valid_input_entity_types and entity_type not in valid_input_entity_types:
            return Response(app.four_o_four, status=404)

        # Initialize a private copy of the config to pass into the transform
        config = _config.CanariConfigParser()
        for p in maltego_request.parameters.values():
            if '.' in p.name:
                config[p.name.replace('.', '/', 1)] = p.value
            else:
                config['plume/%s' % p.name] = p.value
        # The private config variables CANNOT override the server's settings. This is for security?
        config._sections.update(_config.config._sections)

        # Execute it!
        msg = transform(
            maltego_request,
            request_str if hasattr(transform, 'cmd') and callable(transform.cmd) else MaltegoTransformResponseMessage(),
            config
        )

        # Let's serialize the return response and clean up whatever mess was left behind
        if isinstance(msg, MaltegoTransformResponseMessage) or isinstance(msg, basestring):
            return message(msg)
        else:
            raise MaltegoException('Could not resolve message type returned by transform.')

    # Unless we croaked somewhere, then we need to fix things up here...
    except MaltegoException, me:
        return croak(str(me))
    except Exception, e:
        return croak(traceback.format_exc())


# This is where the TDS will ask: "Are you a transform?" and we say "200 - Yes I am!" or "404 - PFO"
@app.route('/<transform_name>', methods=['GET'])
def transform_checker(transform_name):
    if transform_name in app.transforms:
        return Response('Yes?', status=200)
    return Response(app.four_o_four, status=404)

@app.route('/static/<resource_name>', methods=['GET'])
def static_fetcher(resource_name):
    resource_name = 'static/%s' % resource_name
    if resource_name in app.resources:
        return Response(file(resource_name, mode='rb').read(), status=200, mimetype='application/octet-stream')
    return Response(app.four_o_four, status=404)


# This is where we process a transform request.
@app.route('/<transform_name>', methods=['POST'])
def transform_runner(transform_name):
    if transform_name not in app.transforms:
        return Response(app.four_o_four, status=404)
    return dotransform(*app.transforms[transform_name])


# Finally, if you want to run Flask standalone for debugging, just type python plume.py and you're off to the races!
if __name__ == '__main__':
    app.run(debug=True)



#!/usr/bin/env python

import argparse

from common import canari_main, fix_binpath, ParseFieldsAction
from canari.maltego.utils import local_transform_runner, console_message
from framework import SubCommand, Argument
from canari.config import config


__author__ = 'Nadeem Douba'
__copyright__ = 'Copyright 2012, Canari Project'
__credits__ = []

__license__ = 'GPL'
__version__ = '0.7'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'


@SubCommand(
    canari_main,
    help='Runs Canari local transforms in a terminal-friendly fashion.',
    description='Runs Canari local transforms in a terminal-friendly fashion.'
)
@Argument(
    'transform',
    metavar='<transform>',
    help='The name of the transform you wish to run (e.g. sploitego.transforms.nmapfastscan).'
)
@Argument(
    'params',
    metavar='[param1 ... paramN]',
    help='Any extra parameters that can be sent to the local transform.',
    nargs=argparse.ZERO_OR_MORE
)
@Argument(
    'value',
    metavar='<value>',
    help='The value of the input entity being passed into the local transform.'
)
@Argument(
    'fields',
    metavar='[field1=value1...#fieldN=valueN]',
    help='The fields of the input entity being passed into the local transform.',
    default={},
    action=ParseFieldsAction,
    nargs=argparse.OPTIONAL
)  # This parameter will never be consumed because we use a special parser for this transform.
def debug_transform(opts):
    fix_binpath(config['default/path'])
    local_transform_runner(opts.transform, opts.value, opts.fields, opts.params, config, console_message)


# !/usr/bin/env python

from xml.etree.ElementTree import Element, tostring


__author__ = 'Nadeem Douba'
__copyright__ = 'Copyright 2012, Canari Project'
__credits__ = []

__license__ = 'GPL'
__version__ = '0.2'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'

__all__ = [
    'HTML',
    'TABLE',
    'TR',
    'TD',
    'A',
    'IMG',
    'Table'
]


class HTML(Element):
    def __init__(self, tag='html', attrib={}, **extra):
        attrib = attrib.copy()
        attrib.update(extra)
        super(HTML, self).__init__(tag, attrib)

    def __add__(self, other):
        self.append(other)
        return self

    __iadd__ = __add__

    def __sub__(self, other):
        self.remove(other)
        return self

    __isub__ = __sub__

    def __str__(self):
        return tostring(self)


class TABLE(HTML):
    def __init__(self, title="GENERAL INFORMATION", colspan="2", **kwargs):
        super(TABLE, self).__init__(
            "table",
            attrib={
                'width': '100%',
                'border': '1',
                'rules': 'cols',
                'frame': 'box',
                'cellpadding': '2'
            }
        )

        tr = TR()
        self.append(tr)
        tr.append(TD(title, colspan=colspan, css_class=TD.ONE))

        for i in kwargs:
            self.set(i, str(kwargs[i]) if not isinstance(kwargs[i], basestring) else kwargs[i])


class TR(HTML):
    def __init__(self):
        super(TR, self).__init__('tr')


class A(HTML):
    def __init__(self, label, href, **kwargs):
        attrib = {'href': href}
        attrib.update(kwargs)
        super(A, self).__init__(
            'a',
            attrib=attrib
        )
        self.text = label


class IMG(HTML):
    def __init__(self, src, **kwargs):
        attrib = {'src': src}
        attrib.update(kwargs)

        super(IMG, self).__init__(
            'img',
            attrib=attrib
        )


class TD(HTML):
    ONE = "one"
    TWO = "two"
    THREE = "three"
    VALUE = "value"

    def __init__(self, value, css_class=TWO, align="center", **kwargs):
        super(TD, self).__init__(
            'td',
            attrib={
                'class': css_class,
                'align': align,
            }
        )
        self.text = str(value) if not isinstance(value, basestring) else value

        for i in kwargs:
            self.set(i, str(kwargs[i]) if not isinstance(kwargs[i], basestring) else kwargs[i])


class Table(object):
    def __init__(self, columns, title='GENERAL INFORMATION'):
        self._rows = []
        self._title = title
        self._rows.append([TD(c, TD.THREE) for c in columns])

    def addrow(self, columns):
        self._rows.append([TD(c) for c in columns])

    def __str__(self):
        self.table = TABLE(self._title, colspan=len(self._rows[0]))
        for r in self._rows:
            tr = TR()
            c = tr.getchildren()
            c += r
            self.table += tr
        return str(self.table)
